# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import re
import superdesk
import logging
import jinja2.exceptions

from flask import g, render_template_string, current_app as app
from copy import deepcopy
from superdesk.services import BaseService
from superdesk import Resource, Service, config, get_resource_service
from superdesk.utils import SuperdeskBaseEnum, plaintext_filter
from superdesk.resource import build_custom_hateoas
from superdesk.utc import utcnow, local_to_utc, utc_to_local
from superdesk.errors import SuperdeskApiError
from superdesk.metadata.item import metadata_schema, ITEM_STATE, CONTENT_STATE
from superdesk.celery_app import celery
from apps.tasks import apply_onstage_rule
from apps.archive.common import (
    ARCHIVE,
    CUSTOM_HATEOAS,
    item_schema,
    format_dateline_to_locmmmddsrc,
    insert_into_versions,
)
from apps.auth import get_user

from superdesk.lock import lock, unlock
from superdesk.celery_task_utils import get_lock_id
from croniter import croniter
from datetime import datetime
from flask_babel import _
from superdesk.notification import push_notification

CONTENT_TEMPLATE_RESOURCE = "content_templates"
CONTENT_TEMPLATE_PRIVILEGE = CONTENT_TEMPLATE_RESOURCE
KILL_TEMPLATE_NOT_REQUIRED_FIELDS = ["schedule", "dateline", "template_desks", "schedule_desk", "schedule_stage"]
PLAINTEXT_FIELDS = {"headline"}
TEMPLATE_DATA_IGNORE_FIELDS = {  # fields to be ignored when creating item from template
    config.ID_FIELD,
    config.LAST_UPDATED,
    config.DATE_CREATED,
    config.ETAG,
    config.VERSION,
    "task",
    "firstcreated",
    "versioncreated",
    "firstpublished",
    "fields_meta",
}


logger = logging.getLogger(__name__)


class TemplateType(SuperdeskBaseEnum):
    KILL = "kill"
    CREATE = "create"
    HIGHLIGHTS = "highlights"


def get_schema():
    schema = deepcopy(metadata_schema)
    return schema


def get_next_run(schedule, now_utc=None):
    """Get next run time based on schedule.

    Schedule is day of week and time.

    :param dict schedule: dict with `day_of_week` and `create_at` params
    :param now_utc
    :return datetime
    """
    if not schedule.get("is_active", False):
        return None

    if now_utc is None:
        now_utc = utcnow()

    now_utc = now_utc.replace(second=0)

    # Derive the first cron_list entry from the create_at and day_of_week
    if "create_at" in schedule and "cron_list" not in schedule:
        time = schedule.get("create_at").split(":")
        cron_days = ",".join(schedule.get("day_of_week", "*")) if len(schedule.get("day_of_week")) else "*"
        cron_entry = "{} {} * * {}".format(time[1], time[0], cron_days)
        schedule["cron_list"] = [cron_entry]
        schedule.pop("create_at", None)

    # adjust current time to the schedule's timezone
    tz_name = schedule.get("time_zone", "UTC")
    if tz_name != "UTC":
        current_local_datetime = utc_to_local(tz_name, now_utc)  # convert utc to local time
        cron = croniter(schedule.get("cron_list")[0], current_local_datetime)
        next_run = local_to_utc(tz_name, cron.get_next(datetime))
        for cron_entry in schedule.get("cron_list"):
            next_candidate = local_to_utc(tz_name, croniter(cron_entry, current_local_datetime).get_next(datetime))
            if next_candidate < next_run:
                next_run = next_candidate
    else:
        cron = croniter(schedule.get("cron_list")[0], now_utc)
        next_run = cron.get_next(datetime)
        for cron_entry in schedule.get("cron_list"):
            next_candidate = croniter(cron_entry, now_utc).get_next(datetime)
            if next_candidate < next_run:
                next_run = next_candidate

    return next_run


def push_template_notification(docs, event="template:update"):
    user = get_user()
    template_desks = set()

    for doc in docs:
        if doc.get("template_desks"):
            template_desks.update([str(template) for template in doc.get("template_desks")])

    push_notification(event, user=str(user.get(config.ID_FIELD, "")), desks=list(template_desks))


class ContentTemplatesResource(Resource):
    schema = {
        "data": {
            "type": "dict",
            "schema": get_schema(),
        },
        "template_name": {
            "type": "string",
            "unique_template": True,
            "required": True,
        },
        "template_type": {
            "type": "string",
            "required": True,
            "allowed": TemplateType.values(),
            "default": TemplateType.CREATE.value,
        },
        "template_desks": {
            "type": "list",
            "required": False,
            "nullable": True,
            "schema": Resource.rel("desks", embeddable=False, nullable=True),
        },
        "schedule_desk": Resource.rel("desks", embeddable=False, nullable=True),
        "schedule_stage": Resource.rel("stages", embeddable=False, nullable=True),
        "schedule_macro": {"type": "string", "nullable": True},
        "schedule": {
            "type": "dict",
            "schema": {
                "is_active": {"type": "boolean"},
                "create_at": {"type": "string"},
                # List of cron expressions that determine the times items should be created.
                "cron_list": {"type": "list", "required": False, "nullable": True, "schema": {"type": "string"}},
                "day_of_week": {"type": "list"},
                "time_zone": {"type": "string", "nullable": True},
            },
        },
        "last_run": {"type": "datetime", "readonly": True},
        "next_run": {"type": "datetime", "readonly": True},
        "user": Resource.rel("users"),
        "is_public": {"type": "boolean", "unique_template": True, "default": False},
        "item": {
            "type": "dict",
            "schema": {},
            "allow_unknown": True,
        },
    }

    additional_lookup = {"url": 'regex("[\w]+")', "field": "template_name"}

    resource_methods = ["GET", "POST"]
    item_methods = ["GET", "PATCH", "DELETE"]
    no_privileges = True

    merge_nested_documents = True

    mongo_indexes = {
        "user_1_template_name_1_is_public_1": ([("user", 1), ("template_name", 1), ("is_public", 1)], {"unique": True}),
    }


class ContentTemplatesService(BaseService):
    def get(self, req, lookup):
        active_user = g.get("user", {})
        privileges = active_user.get("active_privileges", {})
        if not lookup:
            lookup = {}
        if req and req.args and req.args.get("manage"):
            if not privileges.get("content_templates") and not privileges.get("personal_template"):
                lookup.update({"$and": [{"is_public": False}, {"user": active_user.get("_id")}]})
            elif not privileges.get("content_templates") and privileges.get("personal_template"):
                lookup.update({"$or": [{"is_public": False}, {"user": active_user.get("_id")}]})
            elif privileges.get("content_templates") and not privileges.get("personal_template"):
                lookup.update({"$or": [{"is_public": True}, {"user": active_user.get("_id")}]})
        else:
            if not privileges.get("personal_template"):
                lookup.update({"$or": [{"is_public": True}, {"user": active_user.get("_id")}]})
        results = super().get(req, lookup)

        return results

    def on_create(self, docs):
        for doc in docs:
            self._validate_privileges(doc, action="create")
            doc["template_name"] = doc["template_name"].lower().strip()
            if doc.get("schedule"):
                doc["next_run"] = get_next_run(doc.get("schedule"))

            if doc.get("template_type") == TemplateType.KILL.value and any(
                key for key in doc.keys() if key in KILL_TEMPLATE_NOT_REQUIRED_FIELDS
            ):
                raise SuperdeskApiError.badRequestError(
                    message=_("Invalid kill template. {fields} are not allowed").format(
                        fields=", ".join(KILL_TEMPLATE_NOT_REQUIRED_FIELDS)
                    )
                )
            if doc.get("template_type") == TemplateType.KILL.value:
                self._validate_kill_template(doc)
            if get_user():
                doc.setdefault("user", get_user()[config.ID_FIELD])
            self._validate_template_desks(doc)

    def on_created(self, docs):
        push_template_notification(docs)

    def on_update(self, updates, original):
        self._validate_privileges(original, action="update")
        if (
            updates.get("template_type")
            and updates.get("template_type") != original.get("template_type")
            and updates.get("template_type") == TemplateType.KILL.value
        ):
            self._validate_kill_template(updates)
            self._process_kill_template(updates)

        if updates.get("schedule"):
            original_schedule = deepcopy(original.get("schedule", {}))
            original_schedule.update(updates.get("schedule"))
            updates["next_run"] = get_next_run(original_schedule)
        self._validate_template_desks(updates, original)

        profile_id = updates.get("data", {}).get("profile")
        if profile_id and str(profile_id) != str(original.get("data", {}).get("profile", "")):
            # if profile is changed remove unnecessary fields from template
            original_template = deepcopy(original)
            original_template.update(updates)
            profile = get_resource_service("content_types").find_one(req=None, _id=profile_id)
            data, _ = self._reset_fields(original_template, profile)
            updates["data"] = data

    def on_updated(self, updates, original):
        push_template_notification([updates, original])

    def on_fetched(self, docs):
        self.enhance_items(docs[config.ITEMS])

    def on_fetched_item(self, doc):
        self.enhance_items([doc])

    def enhance_items(self, items):
        for item in items:
            self.handle_existing_data(item)

    def handle_existing_data(self, item):
        schedule = item.get("schedule")
        if schedule and "cron_list" not in schedule and "create_at" in schedule:
            time = schedule.get("create_at").split(":")
            cron_days = ",".join(schedule.get("day_of_week", "*")) if len(schedule.get("day_of_week")) else "*"
            cron_entry = "{} {} * * {}".format(time[1], time[0], cron_days)
            schedule["cron_list"] = [cron_entry]
            schedule.pop("create_at", None)

    def on_delete(self, doc):
        self._validate_privileges(doc, action="delete")
        if doc.get("template_type") == TemplateType.KILL.value:
            raise SuperdeskApiError.badRequestError(_("Kill templates can not be deleted."))

    def on_deleted(self, doc):
        push_template_notification([doc])

    def get_scheduled_templates(self, now):
        """Get the template by schedule

        :param datetime now:
        :return MongoCursor:
        """
        query = {"next_run": {"$lte": now}, "schedule.is_active": True}
        return self.find(query)

    def get_templates_by_profile_id(self, profile_id):
        """Get all templates by profile id"""
        templates = self.get(req=None, lookup=None)
        return [t for t in templates if str(t.get("data", {}).get("profile", "")) == str(profile_id)]

    def update_template_profile(self, updates, profile_id, templates=None):
        """
        Finds the templates that are referencing the given
        content profile an clears the disabled fields
        :param updates: changed data for the content profile
        :param profile_id: id of the profile in string
        :param templates: list of templates to process
        """
        if not templates:
            templates = list(self.get_templates_by_profile_id(profile_id))

        for template in templates:
            data, processed = self._reset_fields(template, updates)
            if processed:
                self.patch(template.get(config.ID_FIELD), {"data": data})

    def _reset_fields(self, template, profile_data):
        """
        Removes fields from template which is disabled or doesn't exist in profile
        """
        fields_to_keep = [
            "profile",
            "type",
            "flags",
            "format",
            "pubstatus",
            "language",
            "usageterms",
            "company_codes",
            "keywords",
            "target_regions",
            "target_types",
            "target_subscribers",
        ]
        data = deepcopy(template.get("data", {}))
        schema = profile_data.get("schema", {})
        processed = False

        # Reset fields that are disabled or doesn't exist in content profile
        fields_to_remove = []
        for field, params in data.items():
            if field not in schema or not schema.get(field) or not schema.get(field, {}).get("enabled", True):
                fields_to_remove.append(field)

        for field in fields_to_remove:
            if field not in fields_to_keep:
                if field in metadata_schema:
                    if metadata_schema.get(field, {}).get("nullable"):
                        data[field] = None
                    else:
                        if metadata_schema.get(field, {}).get("type") == "list":
                            data[field] = []
                        if metadata_schema.get(field, {}).get("type") == "string":
                            data[field] = ""
                        if metadata_schema.get(field, {}).get("type") == "integer":
                            data[field] = 0
                        if metadata_schema.get(field, {}).get("type") == "dict":
                            data[field] = {}
                else:
                    data[field] = None
                processed = True

        return data, processed

    def get_template_by_name(self, template_name):
        """Get the template by name

        :param str template_name: template name
        :return dict: template
        """
        query = {"template_name": re.compile("^{}$".format(template_name), re.IGNORECASE)}
        return self.find_one(req=None, **query)

    def _validate_kill_template(self, doc):
        """
        Validates input values for kill templates
        """
        if doc.get("template_type") != TemplateType.KILL.value:
            return

        if doc.get("template_desks"):
            raise SuperdeskApiError.badRequestError("Kill templates can not be assigned to desks")
        if "is_public" in doc and doc["is_public"] is False:
            raise SuperdeskApiError.badRequestError(_("Kill templates must be public"))
        doc["is_public"] = True

    def _validate_template_desks(self, updates, original=None):
        """
        Validate template desks value
        """
        if original is None:
            original = {}
        template_type = updates.get("template_type", original.get("template_type"))
        if (
            template_type != TemplateType.CREATE.value
            and type(updates.get("template_desks")) == list
            and len(updates["template_desks"]) > 1
        ):
            raise SuperdeskApiError.badRequestError(
                message=_("Templates that are not create type can only be assigned to one desk!")
            )

    def _process_kill_template(self, doc):
        """
        Marks certain field required by the kill as null.
        """
        if doc.get("template_type") != TemplateType.KILL.value:
            return

        for key in KILL_TEMPLATE_NOT_REQUIRED_FIELDS:
            if key in metadata_schema:
                doc.setdefault("data", {})
                doc["data"][key] = None
            else:
                doc[key] = None

    def _validate_privileges(self, doc, action=None):
        active_user = g.get("user")
        user = doc.get("user")
        privileges = active_user.get("active_privileges", {}) if active_user else {}
        if (active_user and active_user.get("user_type")) == "administrator":
            return
        elif (
            active_user
            and user
            and not doc.get("is_public")
            and active_user.get(config.ID_FIELD) != doc.get("user")
            and not privileges.get("personal_template")
        ):
            raise SuperdeskApiError.badRequestError(
                _("You dont have the privilege to {action} another user personal template".format(action=action))
            )
        elif active_user and doc.get("is_public") and not privileges.get(CONTENT_TEMPLATE_PRIVILEGE):
            raise SuperdeskApiError.badRequestError(
                _("You dont have the privilege to {action} the public template".format(action=action))
            )


class ContentTemplatesApplyResource(Resource):
    endpoint_name = "content_templates_apply"
    resource_title = endpoint_name
    schema = {
        "template_name": {"type": "string", "required": True},
        "item": {"type": "dict", "required": True, "schema": item_schema()},
        "_links": {"type": "dict"},
    }

    # in response there can be anything..
    schema.update(get_schema())

    resource_methods = ["POST"]
    item_methods = []
    privileges = {"POST": ARCHIVE}
    url = "content_templates_apply"


class ContentTemplatesApplyService(Service):
    def create(self, docs, **kwargs):
        doc = docs[0] if len(docs) > 0 else {}
        template_name = doc.get("template_name")
        item = doc.get("item") or {}
        item["desk_name"] = get_resource_service("desks").get_desk_name(item.get("task", {}).get("desk"))

        if not template_name:
            SuperdeskApiError.badRequestError(message="Invalid Template Name")

        if not item:
            SuperdeskApiError.badRequestError(message="Invalid Item")

        template = superdesk.get_resource_service("content_templates").get_template_by_name(template_name)
        if not template:
            SuperdeskApiError.badRequestError(message="Invalid Template")

        updates = render_content_template(item, template)
        item.update(updates)

        if template_name == "kill":
            apply_null_override_for_kill(item)

        docs[0] = item
        build_custom_hateoas(CUSTOM_HATEOAS, docs[0])
        return [docs[0].get(config.ID_FIELD)]


def render_content_template_by_name(item, template_name):
    """Apply template by name.

    :param dict item: item on which template is applied
    :param str template_name: template name
    :return dict: updates to the item
    """
    # get the kill template
    template = superdesk.get_resource_service("content_templates").get_template_by_name(template_name)
    if not template:
        SuperdeskApiError.badRequestError(message="{} Template missing.".format(template_name))

    # apply the kill template
    return render_content_template(item, template)


def render_content_template_by_id(item, template_id, update=False):
    """Apply template by name.

    :param dict item: item on which template is applied
    :param str template_id: template _id
    :param bool update: apply updates to item
    :return dict: updates to the item
    """
    # get the kill template
    template = superdesk.get_resource_service("content_templates").find_one(req=None, _id=template_id)
    if not template:
        SuperdeskApiError.badRequestError(message="{} Template missing.".format(template_id))

    return render_content_template(item, template, update)


def render_content_template(item, template, update=False):
    """Render the template.

    :param dict item: item on which template is applied
    :param dict template: template
    :return dict: updates to the item
    """
    new_template_data_ignore_fields = TEMPLATE_DATA_IGNORE_FIELDS.copy()
    kwargs = dict(item=item, user=get_user())
    dateline_present_in_user_preferences = (
        kwargs["user"].get("user_preferences", {}).get("dateline:located", {}).get("located")
    )
    if dateline_present_in_user_preferences:
        new_template_data_ignore_fields.add("dateline")

    template_data = template.get("data", {}) if template else {}

    def render_content_template_fields(data, dest=None, top=True):
        updates = {}
        for key, value in data.items():
            if (top and key in new_template_data_ignore_fields) or not value:
                continue

            if top and key == "extra":
                updates[key] = render_content_template_fields(value, top=False)
                if update:
                    item.setdefault(key, {}).update(updates[key])
            elif isinstance(value, str):
                try:
                    updates[key] = render_template_string(value, **kwargs)
                except jinja2.exceptions.UndefinedError as err:
                    logger.error(err, extra=dict(field=key, template=value))
                except jinja2.exceptions.TemplateSyntaxError as err:
                    logger.error(err, extra=dict(field=key, template=value))
            elif isinstance(value, (dict, list)):
                updates[key] = value
            elif not isinstance(value, (dict, list)):
                updates[key] = value

        if top:
            update_dateline(updates)
            filter_plaintext_fields(updates)

        if update:
            for key, value in updates.items():
                if item.get(key) and isinstance(item[key], dict):
                    item[key].update(value)
                else:
                    item[key] = value
        return updates

    return render_content_template_fields(template_data, dest=item)


def get_scheduled_templates(now):
    """Get templates that should be used to create items for given time.

    :param datetime now
    :return Cursor
    """
    return superdesk.get_resource_service("content_templates").get_scheduled_templates(now)


def set_template_timestamps(template, now):
    """Update template `next_run` field to next time it should run.

    :param dict template
    :param datetime now
    """
    updates = {
        "last_run": now,
        "next_run": get_next_run(template.get("schedule"), now),
    }
    service = superdesk.get_resource_service("content_templates")
    service.update(template[config.ID_FIELD], updates, template)


def get_item_from_template(template):
    """Get item dict using data from template.

    :param dict template
    """
    item = template.get("data", {})
    item[ITEM_STATE] = CONTENT_STATE.SUBMITTED
    if template.get("schedule_desk"):
        item["task"] = {"desk": template["schedule_desk"], "stage": template.get("schedule_stage")}
    item["template"] = template.get("_id")
    item["macro"] = template.get("schedule_macro")
    item.pop("firstcreated", None)
    item.pop("versioncreated", None)

    update_dateline(item)
    filter_plaintext_fields(item)

    return item


def update_dateline(item):
    # handle dateline
    dateline = item.get("dateline", {})
    dateline["date"] = utcnow()
    if dateline.get("located"):
        dateline["text"] = format_dateline_to_locmmmddsrc(dateline["located"], dateline["date"])


def filter_plaintext_fields(item):
    """Filter out html from plaintext fields."""
    for field in PLAINTEXT_FIELDS:
        if field in item:
            item[field] = plaintext_filter(item[field])


def apply_null_override_for_kill(item):
    for key in app.config["KILL_TEMPLATE_NULL_FIELDS"]:
        if key in item:
            item[key] = None


@celery.task(soft_time_limit=120)
def create_scheduled_content(now=None):
    lock_name = get_lock_id("Template", "Schedule")
    if not lock(lock_name, expire=130):
        logger.info("Task: {} is already running.".format(lock_name))
        return

    try:
        if now is None:
            now = utcnow()
        templates = get_scheduled_templates(now)
        production = superdesk.get_resource_service(ARCHIVE)
        items = []
        for template in templates:
            set_template_timestamps(template, now)
            item = get_item_from_template(template)
            item[config.VERSION] = 1
            production.post([item])
            insert_into_versions(doc=item)
            try:
                apply_onstage_rule(item, item.get(config.ID_FIELD))
            except Exception as ex:  # noqa
                logger.exception("Failed to apply on stage rule while scheduling template.")
            items.append(item)
        return items
    except Exception as e:
        logger.exception("Task: {} failed with error {}.".format(lock_name, str(e)))
    finally:
        unlock(lock_name)


def create_template_for_profile(items):
    """Create templates based on given profiles.

    Each template should have same name like profile.

    :param items: list of profiles
    """
    templates = []
    for profile in items:
        if profile.get("label"):
            templates.append(
                {
                    "template_name": profile.get("label"),
                    "is_public": True,
                    "data": {"profile": str(profile.get(config.ID_FIELD))},
                }
            )
    if templates:
        superdesk.get_resource_service(CONTENT_TEMPLATE_RESOURCE).post(templates)


def remove_profile_from_templates(item):
    """Removes the profile data from templates that are using the profile

    :param item: deleted content profile
    """
    templates = list(
        superdesk.get_resource_service(CONTENT_TEMPLATE_RESOURCE).get_templates_by_profile_id(item.get(config.ID_FIELD))
    )
    for template in templates:
        template.get("data", {}).pop("profile", None)
        superdesk.get_resource_service(CONTENT_TEMPLATE_RESOURCE).patch(template[config.ID_FIELD], template)



#!/usr/bin/env python3
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""Unit tests for editor utils"""

import json
import uuid
import unittest
import flask
import superdesk.editor_utils as editor_utils

from superdesk.editor_utils import Editor3Content


class Editor3TestCase(unittest.TestCase):
    def setUp(self):
        self.app = flask.Flask(__name__)
        self.app.app_context().push()
        super().setUp()
        if "EMBED_PRE_PROCESS" in self.app.config:
            del self.app.config["EMBED_PRE_PROCESS"]

    def build_item(self, draftjs_data, field="body_html"):
        return {
            "fields_meta": {
                field: {
                    "draftjsState": [draftjs_data],
                },
            },
        }

    def update_item_field(self, item, draftjs_data, field):
        item["fields_meta"][field] = {"draftjsState": [draftjs_data]}

    def blocks_with_text(self, data_list):
        draftjs_data = {
            "blocks": [],
            "entityMap": {},
        }
        blocks = draftjs_data["blocks"]
        for item_data in data_list:
            type_, depth, text = item_data
            blocks.append(
                {
                    "key": str(uuid.uuid4()),
                    "text": text,
                    "type": type_,
                    "depth": depth,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                }
            )
        return draftjs_data

    def create_table(self, cols, rows, cells, with_header=False):
        cells_list = []
        draftjs_data = {
            "blocks": [
                {
                    "key": "first_block",
                    "text": "",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                },
                {
                    "key": "table_key",
                    "text": " ",
                    "type": "atomic",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [{"offset": 0, "length": 1, "key": 0}],
                    "data": {},
                },
                {
                    "key": "last_block",
                    "text": "",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                },
            ],
            "entityMap": {
                "0": {
                    "type": "TABLE",
                    "mutability": "MUTABLE",
                    "data": {
                        "data": {
                            "cells": cells_list,
                            "numCols": cols,
                            "numRows": rows,
                            "withHeader": with_header,
                        }
                    },
                }
            },
        }

        for cell_row in cells:
            row = []
            cells_list.append(row)
            for cell in cell_row:
                row.append(
                    {
                        "blocks": [
                            {
                                "key": str(uuid.uuid4()),
                                "text": cell,
                                "type": "unstyled",
                                "depth": 0,
                                "inlineStyleRanges": [],
                                "entityRanges": [],
                                "data": {},
                            }
                        ],
                        "entityMap": {},
                    }
                    if cell
                    else None
                )

        draftjs_data["blocks"][0]["data"]["data"] = json.dumps(draftjs_data["entityMap"]["0"]["data"]["data"])
        return draftjs_data

    def test_no_formatting(self):
        """Check that a sentence without formatting can generate HTML"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": 'The name of Highlaws comes from the Old English hēah-hlāw, meaning "high mounds".',
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                }
            ],
            "entityMap": {},
        }
        expected = '<p>The name of Highlaws comes from the Old English hēah-hlāw, meaning "high mounds".</p>'

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_simple_inline_styles(self):
        """Check that a sentence with simple inline styles can generate HTML"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": 'The name of Highlaws comes from the Old English hēah-hlāw, meaning "high mounds".',
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [
                        {"offset": 0, "length": 3, "style": "BOLD"},
                        {"offset": 4, "length": 4, "style": "ITALIC"},
                        {"offset": 9, "length": 2, "style": "UNDERLINE"},
                        {"offset": 12, "length": 8, "style": "STRIKETHROUGH"},
                        {"offset": 21, "length": 5, "style": "SUBSCRIPT"},
                        {"offset": 27, "length": 4, "style": "SUPERSCRIPT"},
                    ],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                }
            ],
            "entityMap": {},
        }

        expected = (
            "<p><b>The</b> <i>name</i> <u>of</u> <s>Highlaws</s> <sub>comes</sub> <sup>from</sup> the Old English "
            'hēah-hlāw, meaning "high mounds".</p>'
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_heading(self):
        """Check that headlines are converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": "The name of Highlaws comes from the Old English hēah-hlāw",
                    "type": "header-one",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                },
                {
                    "key": "32mrs",
                    "text": ', meaning "high mounds". In the past,',
                    "type": "header-two",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
                {
                    "key": "d2ggv",
                    "text": "variant spellings included Heelawes, Hielawes,",
                    "type": "header-three",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
                {
                    "key": "3eqv5",
                    "text": "Highlows, Hielows, and Hylaws.",
                    "type": "header-four",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
                {
                    "key": "8lcpm",
                    "text": "[2] The hamlet appears in a survey of Holm Cultram dating back",
                    "type": "header-five",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
                {
                    "key": "emuqc",
                    "text": "to the year 1538, during the reign of Henry VIII.",
                    "type": "header-six",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
                {
                    "key": "90o9n",
                    "text": "There were at least thirteen families resident in Highlaws at that time.[3] Abdastartus "
                    "is a genus of lace bugs in the family Tingidae. There are about five described species in "
                    "Abdastartus.",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
            ],
            "entityMap": {},
        }

        expected = (
            '<h1>The name of Highlaws comes from the Old English hēah-hlāw</h1><h2>, meaning "high mounds". In the past'
            ",</h2><h3>variant spellings included Heelawes, Hielawes,</h3><h4>Highlows, Hielows, and Hylaws.</h4><h5>["
            "2] The hamlet appears in a survey of Holm Cultram dating back</h5><h6>to the year 1538, during the reign o"
            "f Henry VIII.</h6><p>There were at least thirteen families resident in Highlaws at that time.[3] Abdastart"
            "us is a genus of lace bugs in the family Tingidae. There are about five described species in Abdastartus.<"
            "/p>"
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_blockquote(self):
        """Check that blockqotes are converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": 'The name of Highlaws comes from the Old English hēah-hlāw, meaning "high mounds".',
                    "type": "blockquote",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                },
                {
                    "key": "2u79k",
                    "text": "In the past, variant spellings included Heelawes, Hielawes, Highlows, Hielows, and "
                    "Hylaws.",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
            ],
            "entityMap": {},
        }

        expected = (
            '<blockquote>The name of Highlaws comes from the Old English hēah-hlāw, meaning "high mounds".</blockquote>'
            "<p>In the past, variant spellings included Heelawes, Hielawes, Highlows, Hielows, and Hylaws.</p>"
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_code_blocks(self):
        """Check that code blocks are converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": 'The name of Highlaws comes from the Old English hēah-hlāw, meaning "high mounds".',
                    "type": "code-block",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                },
                {
                    "key": "2u79k",
                    "text": "In the past, variant spellings included Heelawes, Hielawes, Highlows, Hielows, and "
                    "Hylaws.",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
            ],
            "entityMap": {},
        }

        expected = (
            '<pre><code>The name of Highlaws comes from the Old English hēah-hlāw, meaning "high mounds".</code></pre><'
            "p>In the past, variant spellings included Heelawes, Hielawes, Highlows, Hielows, and Hylaws.</p>"
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_links(self):
        """Check that links are converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": 'The name of Highlaws comes from the Old English hēah-hlāw, meaning "high mounds".',
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [{"offset": 12, "length": 8, "key": 0}],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                }
            ],
            "entityMap": {
                "0": {
                    "type": "LINK",
                    "mutability": "MUTABLE",
                    "data": {"link": {"href": "https://en.wikipedia.org/wiki/Highlaws"}},
                }
            },
        }

        expected = (
            '<p>The name of <a href="https://en.wikipedia.org/wiki/Highlaws">Highlaws</a> comes from the Old English hē'
            'ah-hlāw, meaning "high mounds".</p>'
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_simple_table(self):
        """Check that a simple table is converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": "",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                },
                {
                    "key": "fhokc",
                    "text": " ",
                    "type": "atomic",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [{"offset": 0, "length": 1, "key": 0}],
                    "data": {
                        "data": '{"cells":[[{"blocks":[{"key":"k8sb","text":"three","type":"unstyled","depth":0,"inline'
                        'StyleRanges":[],"entityRanges":[],"data":{}}],"entityMap":{}},{"blocks":[{"key":"a25i9'
                        '","text":"column","type":"unstyled","depth":0,"inlineStyleRanges":[],"entityRanges":[]'
                        ',"data":{}}],"entityMap":{}},{"blocks":[{"key":"ej3lv","text":"table","type":"unstyled'
                        '","depth":0,"inlineStyleRanges":[],"entityRanges":[],"data":{}}],"entityMap":{}}],[{"b'
                        'locks":[{"key":"f0qc0","text":"example","type":"unstyled","depth":0,"inlineStyleRanges'
                        '":[],"entityRanges":[],"data":{}}],"entityMap":{}},{"blocks":[{"key":"50s2o","text":"r'
                        'ight","type":"unstyled","depth":0,"inlineStyleRanges":[],"entityRanges":[],"data":{}}]'
                        ',"entityMap":{}},{"blocks":[{"key":"escgd","text":"here","type":"unstyled","depth":0,"'
                        'inlineStyleRanges":[],"entityRanges":[],"data":{}}],"entityMap":{}}]],"numRows":2,"num'
                        'Cols":3,"withHeader":false}'
                    },
                },
                {
                    "key": "2u79k",
                    "text": "",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
            ],
            "entityMap": {
                "0": {
                    "type": "TABLE",
                    "mutability": "MUTABLE",
                    "data": {
                        "data": {
                            "cells": [
                                [
                                    {
                                        "blocks": [
                                            {
                                                "key": "k8sb",
                                                "text": "three",
                                                "type": "unstyled",
                                                "depth": 0,
                                                "inlineStyleRanges": [],
                                                "entityRanges": [],
                                                "data": {},
                                            }
                                        ],
                                        "entityMap": {},
                                    },
                                    {
                                        "blocks": [
                                            {
                                                "key": "a25i9",
                                                "text": "column",
                                                "type": "unstyled",
                                                "depth": 0,
                                                "inlineStyleRanges": [],
                                                "entityRanges": [],
                                                "data": {},
                                            }
                                        ],
                                        "entityMap": {},
                                    },
                                    {
                                        "blocks": [
                                            {
                                                "key": "ej3lv",
                                                "text": "table",
                                                "type": "unstyled",
                                                "depth": 0,
                                                "inlineStyleRanges": [],
                                                "entityRanges": [],
                                                "data": {},
                                            }
                                        ],
                                        "entityMap": {},
                                    },
                                ],
                                [
                                    {
                                        "blocks": [
                                            {
                                                "key": "f0qc0",
                                                "text": "example",
                                                "type": "unstyled",
                                                "depth": 0,
                                                "inlineStyleRanges": [],
                                                "entityRanges": [],
                                                "data": {},
                                            }
                                        ],
                                        "entityMap": {},
                                    },
                                    {
                                        "blocks": [
                                            {
                                                "key": "50s2o",
                                                "text": "right",
                                                "type": "unstyled",
                                                "depth": 0,
                                                "inlineStyleRanges": [],
                                                "entityRanges": [],
                                                "data": {},
                                            }
                                        ],
                                        "entityMap": {},
                                    },
                                    {
                                        "blocks": [
                                            {
                                                "key": "escgd",
                                                "text": "here",
                                                "type": "unstyled",
                                                "depth": 0,
                                                "inlineStyleRanges": [],
                                                "entityRanges": [],
                                                "data": {},
                                            }
                                        ],
                                        "entityMap": {},
                                    },
                                ],
                            ],
                            "numRows": 2,
                            "numCols": 3,
                            "withHeader": False,
                        }
                    },
                }
            },
        }

        expected = (
            "<table><tbody><tr><td><p>three</p></td><td><p>column</p></td><td><p>table</p></td></tr><tr><td><p>example<"
            "/p></td><td><p>right</p></td><td><p>here</p></td></tr></tbody></table>"
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_table_inline_styles(self):
        """Check that tables with inline styles are converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": "",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                },
                {
                    "key": "fhokc",
                    "text": " ",
                    "type": "atomic",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [{"offset": 0, "length": 1, "key": 0}],
                    "data": {
                        "data": '{"cells":[[{"blocks":[{"key":"k8sb","text":"three","type":"unstyled","depth":0,"inline'
                        'StyleRanges":[{"offset":0,"length":5,"style":"BOLD"}],"entityRanges":[],"data":{}}],"e'
                        'ntityMap":{}},{"blocks":[{"key":"a25i9","text":"column","type":"unstyled","depth":0,"i'
                        'nlineStyleRanges":[{"offset":0,"length":6,"style":"ITALIC"}],"entityRanges":[],"data":'
                        '{}}],"entityMap":{}},{"blocks":[{"key":"ej3lv","text":"table","type":"unstyled","depth'
                        '":0,"inlineStyleRanges":[{"offset":0,"length":5,"style":"UNDERLINE"}],"entityRanges":['
                        '],"data":{}}],"entityMap":{}}],[{"blocks":[{"key":"f0qc0","text":"example","type":"uns'
                        'tyled","depth":0,"inlineStyleRanges":[{"offset":0,"length":7,"style":"SUBSCRIPT"}],"en'
                        'tityRanges":[],"data":{}}],"entityMap":{}},{"blocks":[{"key":"50s2o","text":"right","t'
                        'ype":"unstyled","depth":0,"inlineStyleRanges":[{"offset":0,"length":5,"style":"SUPERSC'
                        'RIPT"}],"entityRanges":[],"data":{}}],"entityMap":{}},{"blocks":[{"key":"escgd","text"'
                        ':"here","type":"unstyled","depth":0,"inlineStyleRanges":[{"offset":0,"length":4,"style'
                        '":"STRIKETHROUGH"}],"entityRanges":[],"data":{}}],"entityMap":{}}]],"numRows":2,"numCo'
                        'ls":3,"withHeader":false}'
                    },
                },
                {
                    "key": "2u79k",
                    "text": "",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
            ],
            "entityMap": {
                "0": {
                    "type": "TABLE",
                    "mutability": "MUTABLE",
                    "data": {
                        "data": {
                            "cells": [
                                [
                                    {
                                        "blocks": [
                                            {
                                                "key": "k8sb",
                                                "text": "three",
                                                "type": "unstyled",
                                                "depth": 0,
                                                "inlineStyleRanges": [{"offset": 0, "length": 5, "style": "BOLD"}],
                                                "entityRanges": [],
                                                "data": {},
                                            }
                                        ],
                                        "entityMap": {},
                                    },
                                    {
                                        "blocks": [
                                            {
                                                "key": "a25i9",
                                                "text": "column",
                                                "type": "unstyled",
                                                "depth": 0,
                                                "inlineStyleRanges": [{"offset": 0, "length": 6, "style": "ITALIC"}],
                                                "entityRanges": [],
                                                "data": {},
                                            }
                                        ],
                                        "entityMap": {},
                                    },
                                    {
                                        "blocks": [
                                            {
                                                "key": "ej3lv",
                                                "text": "table",
                                                "type": "unstyled",
                                                "depth": 0,
                                                "inlineStyleRanges": [{"offset": 0, "length": 5, "style": "UNDERLINE"}],
                                                "entityRanges": [],
                                                "data": {},
                                            }
                                        ],
                                        "entityMap": {},
                                    },
                                ],
                                [
                                    {
                                        "blocks": [
                                            {
                                                "key": "f0qc0",
                                                "text": "example",
                                                "type": "unstyled",
                                                "depth": 0,
                                                "inlineStyleRanges": [{"offset": 0, "length": 7, "style": "SUBSCRIPT"}],
                                                "entityRanges": [],
                                                "data": {},
                                            }
                                        ],
                                        "entityMap": {},
                                    },
                                    {
                                        "blocks": [
                                            {
                                                "key": "50s2o",
                                                "text": "right",
                                                "type": "unstyled",
                                                "depth": 0,
                                                "inlineStyleRanges": [
                                                    {"offset": 0, "length": 5, "style": "SUPERSCRIPT"}
                                                ],
                                                "entityRanges": [],
                                                "data": {},
                                            }
                                        ],
                                        "entityMap": {},
                                    },
                                    {
                                        "blocks": [
                                            {
                                                "key": "escgd",
                                                "text": "here",
                                                "type": "unstyled",
                                                "depth": 0,
                                                "inlineStyleRanges": [
                                                    {"offset": 0, "length": 4, "style": "STRIKETHROUGH"}
                                                ],
                                                "entityRanges": [],
                                                "data": {},
                                            }
                                        ],
                                        "entityMap": {},
                                    },
                                ],
                            ],
                            "numRows": 2,
                            "numCols": 3,
                            "withHeader": False,
                        }
                    },
                }
            },
        }

        expected = (
            "<table><tbody><tr><td><p><b>three</b></p></td><td><p><i>column</i></p></td><td><p><u>table</u></p></td></t"
            "r><tr><td><p><sub>example</sub></p></td><td><p><sup>right</sup></p></td><td><p><s>here</s></p></td></tr></"
            "tbody></table>"
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_table_none(self):
        """Check that a table with None value in a cell is generated correctly"""

        draftjs_data = self.create_table(
            cols=3,
            rows=2,
            cells=[
                ["a", None, "c"],
                ["d", "e", "f"],
            ],
        )

        expected = (
            "<table><tbody><tr><td><p>a</p></td><td></td><td><p>c</p></td></tr><tr><td><p>d</p></td><td><p>e</p></td><t"
            "d><p>f</p></td></tr></tbody></table>"
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_table_single(self):
        """Check that a table with a single row is generated correctly"""

        draftjs_data = self.create_table(
            cols=3,
            rows=1,
            cells=[
                ["a", "b", "c"],
            ],
        )

        expected = "<table><tbody><tr><td><p>a</p></td><td><p>b</p></td><td><p>c</p></td></tr></tbody></table>"

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_table_header(self):
        """Check that a table with header is generated correctly"""

        draftjs_data = self.create_table(
            cols=3,
            rows=3,
            with_header=True,
            cells=[
                ["a", None, "c"],
                ["d", "e", "f"],
                ["g", "h", "i"],
            ],
        )

        expected = (
            "<table><thead><tr><th><p>a</p></th><th></th><th><p>c</p></th></tr></thead><tbody><tr><td><p>d</p></td><td>"
            "<p>e</p></td><td><p>f</p></td></tr><tr><td><p>g</p></td><td><p>h</p></td><td><p>i</p></td></tr></tbody></t"
            "able>"
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_table_header_single_row(self):
        """Check that a table with header and single row is generated correctly"""

        draftjs_data = self.create_table(
            cols=3,
            rows=1,
            with_header=True,
            cells=[
                ["a", "b", "c"],
            ],
        )

        expected = "<table><thead><tr><th><p>a</p></th><th><p>b</p></th><th><p>c</p></th></tr></thead></table>"

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_table_empty(self):
        """Check that an empty table is generated correctly"""

        draftjs_data = self.create_table(
            cols=3,
            rows=2,
            cells=[],
        )

        expected = (
            "<table><tbody><tr><td></td><td></td><td></td></tr><tr><td></td><td></td><td></td></tr></tbody></table>"
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_image(self):
        """Check that images are converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": "",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                },
                {
                    "key": "fi1d",
                    "text": " ",
                    "type": "atomic",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [{"offset": 0, "length": 1, "key": 0}],
                    "data": {},
                },
                {
                    "key": "60vvd",
                    "text": "",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
            ],
            "entityMap": {
                "0": {
                    "type": "MEDIA",
                    "mutability": "MUTABLE",
                    "data": {
                        "media": {
                            "flags": {
                                "marked_for_not_publication": False,
                                "marked_for_sms": False,
                                "marked_archived_only": False,
                                "marked_for_legal": False,
                            },
                            "language": "en",
                            "_updated": "2019-03-28T17:26:54+0000",
                            "description_text": "pin dec",
                            "source": "Superdesk",
                            "type": "picture",
                            "priority": 6,
                            "_current_version": 2,
                            "versioncreated": "2019-03-28T17:26:54+0000",
                            "task": {
                                "stage": "5c374805149f116db6aae6ad",
                                "user": "5acb79292e03ed5d2a84bbd6",
                                "desk": "5c374805149f116db6aae6af",
                            },
                            "urgency": 3,
                            "alt_text": "pin alt",
                            "_created": "2019-03-28T17:26:53+0000",
                            "genre": [{"name": "Article (news)", "qcode": "Article"}],
                            "guid": "tag:localhost:5000:2019:9df37ab0-4d96-4f95-8da5-06b744ef8604",
                            "renditions": {
                                "baseImage": {
                                    "width": 933,
                                    "mimetype": "image/jpeg",
                                    "href": "http://localhost:5000/api/upload-raw/5c9d03de149f116747b67317.jpg",
                                    "media": "5c9d03de149f116747b67317",
                                    "height": 1400,
                                },
                                "thumbnail": {
                                    "width": 79,
                                    "mimetype": "image/jpeg",
                                    "href": "http://localhost:5000/api/upload-raw/5c9d03de149f116747b67319.jpg",
                                    "media": "5c9d03de149f116747b67319",
                                    "height": 120,
                                },
                                "viewImage": {
                                    "width": 426,
                                    "mimetype": "image/jpeg",
                                    "href": "http://localhost:5000/api/upload-raw/5c9d03de149f116747b6731b.jpg",
                                    "media": "5c9d03de149f116747b6731b",
                                    "height": 640,
                                },
                                "original": {
                                    "width": 3648,
                                    "mimetype": "image/jpeg",
                                    "href": "http://localhost:5000/api/upload-raw/5c9d03dd149f116747b6730f.jpg",
                                    "media": "5c9d03dd149f116747b6730f",
                                    "height": 5472,
                                },
                            },
                            "state": "in_progress",
                            "expiry": None,
                            "byline": None,
                            "headline": "A picture of pineapple",
                            "_etag": "a2c443900319548148e9073b9c5cc4c76c9331c5",
                            "_id": "urn:newsml:localhost:5000:2019-03-28T18:26:53.805271:f47e37fd-a6bd-4a5e-b91d-01b581"
                            "f04e8c",
                            "_type": "archive",
                            "_links": {
                                "self": {
                                    "title": "Archive",
                                    "href": "archive/urn:newsml:localhost:5000:2019-03-28T18:26:53.805271:f47e37fd-a6bd"
                                    "-4a5e-b91d-01b581f04e8c",
                                }
                            },
                            "_latest_version": 2,
                            "selected": False,
                        }
                    },
                }
            },
        }

        expected = (
            '<!-- EMBED START Image {id: "editor_0"} -->\n'
            '<figure><img alt="pin alt" src="http://localhost:5000/api/upload-raw/5c9d03dd149f116747b6730f.jpg">'
            "<figcaption>pin dec</figcaption></figure>\n"
            '<!-- EMBED END Image {id: "editor_0"} -->'
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_image_without_alt_and_desc(self):
        """Check that images without alt text and description are converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": "",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                },
                {
                    "key": "fi1d",
                    "text": " ",
                    "type": "atomic",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [{"offset": 0, "length": 1, "key": 0}],
                    "data": {},
                },
                {
                    "key": "60vvd",
                    "text": "",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
            ],
            "entityMap": {
                "0": {
                    "type": "MEDIA",
                    "mutability": "MUTABLE",
                    "data": {
                        "media": {
                            "flags": {
                                "marked_for_not_publication": False,
                                "marked_for_sms": False,
                                "marked_archived_only": False,
                                "marked_for_legal": False,
                            },
                            "language": "en",
                            "_updated": "2019-03-28T17:26:54+0000",
                            "source": "Superdesk",
                            "type": "picture",
                            "priority": 6,
                            "_current_version": 2,
                            "versioncreated": "2019-03-28T17:26:54+0000",
                            "task": {
                                "stage": "5c374805149f116db6aae6ad",
                                "user": "5acb79292e03ed5d2a84bbd6",
                                "desk": "5c374805149f116db6aae6af",
                            },
                            "urgency": 3,
                            "_created": "2019-03-28T17:26:53+0000",
                            "genre": [{"name": "Article (news)", "qcode": "Article"}],
                            "guid": "tag:localhost:5000:2019:9df37ab0-4d96-4f95-8da5-06b744ef8604",
                            "renditions": {
                                "baseImage": {
                                    "width": 933,
                                    "mimetype": "image/jpeg",
                                    "href": "http://localhost:5000/api/upload-raw/5c9d03de149f116747b67317.jpg",
                                    "media": "5c9d03de149f116747b67317",
                                    "height": 1400,
                                },
                                "thumbnail": {
                                    "width": 79,
                                    "mimetype": "image/jpeg",
                                    "href": "http://localhost:5000/api/upload-raw/5c9d03de149f116747b67319.jpg",
                                    "media": "5c9d03de149f116747b67319",
                                    "height": 120,
                                },
                                "viewImage": {
                                    "width": 426,
                                    "mimetype": "image/jpeg",
                                    "href": "http://localhost:5000/api/upload-raw/5c9d03de149f116747b6731b.jpg",
                                    "media": "5c9d03de149f116747b6731b",
                                    "height": 640,
                                },
                                "original": {
                                    "width": 3648,
                                    "mimetype": "image/jpeg",
                                    "href": "http://localhost:5000/api/upload-raw/5c9d03dd149f116747b6730f.jpg",
                                    "media": "5c9d03dd149f116747b6730f",
                                    "height": 5472,
                                },
                            },
                            "state": "in_progress",
                            "expiry": None,
                            "byline": None,
                            "headline": "A picture of pineapple",
                            "_etag": "a2c443900319548148e9073b9c5cc4c76c9331c5",
                            "_id": "urn:newsml:localhost:5000:2019-03-28T18:26:53.805271:f47e37fd-a6bd-4a5e-b91d-01b581"
                            "f04e8c",
                            "_type": "archive",
                            "_links": {
                                "self": {
                                    "title": "Archive",
                                    "href": "archive/urn:newsml:localhost:5000:2019-03-28T18:26:53.805271:f47e37fd-a6bd"
                                    "-4a5e-b91d-01b581f04e8c",
                                }
                            },
                            "_latest_version": 2,
                            "selected": False,
                        }
                    },
                }
            },
        }

        expected = (
            '<!-- EMBED START Image {id: "editor_0"} -->\n'
            '<figure><img alt="" src="http://localhost:5000/api/upload-raw/5c9d03dd149f116747b6730f.jpg">'
            "</figure>\n"
            '<!-- EMBED END Image {id: "editor_0"} -->'
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_unordered_list(self):
        """Check that unordered lists are converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": "The name of",
                    "type": "unordered-list-item",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                },
                {
                    "key": "fgkp6",
                    "text": "Highlaws comes",
                    "type": "unordered-list-item",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
                {
                    "key": "6g9h6",
                    "text": "from the Old English",
                    "type": "unordered-list-item",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
                {
                    "key": "7v7b1",
                    "text": "hēah-hlāw",
                    "type": "unordered-list-item",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
            ],
            "entityMap": {},
        }

        expected = "<ul><li>The name of</li><li>Highlaws comes</li><li>from the Old English</li><li>hēah-hlāw</li></ul>"

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_ordered_list(self):
        """Check that ordered lists are converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": "The name of",
                    "type": "ordered-list-item",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                },
                {
                    "key": "fgkp6",
                    "text": "Highlaws comes",
                    "type": "ordered-list-item",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
                {
                    "key": "6g9h6",
                    "text": "from the Old English",
                    "type": "ordered-list-item",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
                {
                    "key": "7v7b1",
                    "text": "hēah-hlāw",
                    "type": "ordered-list-item",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
            ],
            "entityMap": {},
        }

        expected = "<ol><li>The name of</li><li>Highlaws comes</li><li>from the Old English</li><li>hēah-hlāw</li></ol>"

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_mixing_ordered_unordered_lists(self):
        """Check that a mix of ordered and unordered lists are converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": "The name of Highlaws comes from the Old English",
                    "type": "ordered-list-item",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                },
                {
                    "key": "6oe0d",
                    "text": 'hēah-hlāw, meaning "high mounds". In the past, variant',
                    "type": "unordered-list-item",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
                {
                    "key": "aegg",
                    "text": "spellings included Heelawes, Hielawes, Highlows,",
                    "type": "ordered-list-item",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
                {
                    "key": "a1qv7",
                    "text": "Hielows, and Hylaws.",
                    "type": "ordered-list-item",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
            ],
            "entityMap": {},
        }

        expected = (
            '<ol><li>The name of Highlaws comes from the Old English</li></ol><ul><li>hēah-hlāw, meaning "high mounds".'
            " In the past, variant</li></ul><ol><li>spellings included Heelawes, Hielawes, Highlows,</li><li>Hielows, a"
            "nd Hylaws.</li></ol>"
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_embed(self):
        """Check that an embed is converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": "",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                },
                {
                    "key": "2s495",
                    "text": " ",
                    "type": "atomic",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [{"offset": 0, "length": 1, "key": 0}],
                    "data": {},
                },
                {
                    "key": "egj1u",
                    "text": "",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {},
                },
            ],
            "entityMap": {
                "0": {
                    "type": "EMBED",
                    "mutability": "MUTABLE",
                    "data": {
                        "data": {
                            "url": "https://www.youtube.com/watch?v=G5-KJgVsoUM",
                            "type": "video",
                            "version": "1.0",
                            "title": "Mother Mother - It's Alright",
                            "author": "MotherMotherVEVO",
                            "author_url": "https://www.youtube.com/channel/UCVzJrFuVWzf8mPiuV3o2_pQ",
                            "provider_name": "YouTube",
                            "description": "Music video by Mother Mother performing It's Alright. © 2019 Mother Mother Music Inc.,"
                            " under exclusive license to Universal Music Canada Inc.\\n\\nhttp://vevo.ly/V49vkg\n",
                            "thumbnail_url": "https://i.ytimg.com/vi/G5-KJgVsoUM/maxresdefault.jpg",
                            "thumbnail_width": 1280,
                            "thumbnail_height": 720,
                            "html": '<div><div style="left: 0; width: 100%; height: 0; position: relative; padding-bottom: '
                            '56.2493%;"><iframe src="//cdn.iframe.ly/api/iframe?url=https%3A%2F%2Fwww.youtube.com%2'
                            'Fwatch%3Fv%3DG5-KJgVsoUM&amp;key=87ca3314a9fa775b5c3a7726100694b0" style="border: 0; t'
                            'op: 0; left: 0; width: 100%; height: 100%; position: absolute;" allowfullscreen scroll'
                            'ing="no" allow="autoplay; encrypted-media"></iframe></div></div>',
                            "cache_age": 86400,
                        }
                    },
                }
            },
        }

        expected = (
            '<div class="embed-block"><div><div style="left: 0; width: 100%; height: 0; position: relative; padding-bot'
            'tom: 56.2493%;"><iframe src="//cdn.iframe.ly/api/iframe?url=https%3A%2F%2Fwww.youtube.com%2Fwatch%3Fv%3DG5'
            '-KJgVsoUM&amp;key=87ca3314a9fa775b5c3a7726100694b0" style="border: 0; top: 0; left: 0; width: 100%; height'
            ': 100%; position: absolute;" allowfullscreen scrolling="no" allow="autoplay; encrypted-media"></iframe></d'
            "iv></div></div>"
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_nested_inline_styles(self):
        """Check that nested inline styles are converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": 'The name of Highlaws comes from the Old English hēah-hlāw, meaning "high mounds". In the past,'
                    " variant spellings included Heelawes, Hielawes, Highlows, Hielows, and Hylaws.",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [
                        {"offset": 4, "length": 43, "style": "BOLD"},
                        {"offset": 21, "length": 5, "style": "UNDERLINE"},
                        {"offset": 32, "length": 3, "style": "ITALIC"},
                    ],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                }
            ],
            "entityMap": {},
        }

        expected = (
            "<p>The <b>name of Highlaws </b><b><u>comes</u></b><b> from </b><b><i>the</i></b><b> Old English</b> hēah-h"
            'lāw, meaning "high mounds". In the past, variant spellings included Heelawes, Hielawes, Highlows, Hielows,'
            " and Hylaws.</p>"
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_overlapping_inline(self):
        """Check that overlapping inline styles are converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": 'The name of Highlaws comes from the Old English hēah-hlāw, meaning "high mounds". In the past,'
                    " variant spellings included Heelawes, Hielawes, Highlows, Hielows, and Hylaws.",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [
                        {"offset": 0, "length": 66, "style": "BOLD"},
                        {"offset": 4, "length": 89, "style": "ITALIC"},
                        {"offset": 27, "length": 142, "style": "UNDERLINE"},
                        {"offset": 165, "length": 5, "style": "SUPERSCRIPT"},
                    ],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                }
            ],
            "entityMap": {},
        }

        expected = (
            "<p><b>The </b><b><i>name of Highlaws comes </i></b><b><i><u>from the Old English hēah-hlāw, meaning</u></i"
            '></b><i><u> "high mounds". In the past</u></i><u>, variant spellings included Heelawes, Hielawes, Highlows'
            ", Hielows, and </u><sup><u>Hyla</u></sup><sup>w</sup>s.</p>"
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_overlapping_inline_with_links(self):
        """Check that overlapping inline styles with links are converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": 'The name of Highlaws comes from the Old English hēah-hlāw, meaning "high mounds". In the past,'
                    " variant spellings included Heelawes, Hielawes, Highlows, Hielows, and Hylaws.",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [
                        {"offset": 12, "length": 14, "style": "BOLD"},
                        {"offset": 55, "length": 6, "style": "BOLD"},
                        {"offset": 48, "length": 18, "style": "UNDERLINE"},
                    ],
                    "entityRanges": [
                        {"offset": 21, "length": 10, "key": 0},
                        {"offset": 36, "length": 21, "key": 1},
                    ],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                }
            ],
            "entityMap": {
                "0": {
                    "type": "LINK",
                    "mutability": "MUTABLE",
                    "data": {"link": {"href": "https://en.wikipedia.org/wiki/Highlaws"}},
                },
                "1": {
                    "type": "LINK",
                    "mutability": "MUTABLE",
                    "data": {"link": {"href": "https://en.wikipedia.org/wiki/Highlaws"}},
                },
            },
        }

        expected = (
            '<p>The name of <b>Highlaws </b><a href="https://en.wikipedia.org/wiki/Highlaws"><b>comes</b> from</a> the '
            '<a href="https://en.wikipedia.org/wiki/Highlaws">Old English <u>hēah-hl</u><b><u>āw</u></b></a><b><u>, me<'
            '/u></b><u>aning</u> "high mounds". In the past, variant spellings included Heelawes, Hielawes, Highlows, H'
            "ielows, and Hylaws.</p>"
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_attachment(self):
        """Check that attachments are converted to HTML correctly"""

        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": 'The name of Highlaws comes from the Old English hēah-hlāw, meaning "high mounds".',
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [{"offset": 12, "length": 8, "key": 0}],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                }
            ],
            "entityMap": {
                "0": {
                    "type": "LINK",
                    "mutability": "MUTABLE",
                    "data": {"link": {"attachment": "5c9dd26d149f114c61d84db0"}},
                }
            },
        }

        expected = (
            '<p>The name of <a data-attachment="5c9dd26d149f114c61d84db0">Highlaws</a> comes from the Old English hēah-'
            'hlāw, meaning "high mounds".</p>'
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_lists(self):
        """Check that lists are correctly generated"""

        draftjs_data = self.blocks_with_text(
            [
                ["unordered-list-item", 0, "1"],
                ["unordered-list-item", 0, "2"],
                ["unordered-list-item", 1, "11"],
                ["unordered-list-item", 1, "22"],
                ["unordered-list-item", 1, "3"],
                ["unordered-list-item", 2, "4"],
                ["unordered-list-item", 2, "5"],
                ["unordered-list-item", 3, "6"],
                ["unordered-list-item", 3, "6.5"],
                ["unordered-list-item", 2, "x"],
                ["unordered-list-item", 1, "7"],
                ["unordered-list-item", 1, "33"],
                ["unordered-list-item", 0, "8"],
            ]
        )

        expected = (
            "<ul><li>1</li><li>2<ul><li>11</li><li>22</li><li>3<ul><li>4</li><li>5<ul><li>6</li><li>6.5</li></ul></li><"
            "li>x</li></ul></li><li>7</li><li>33</li></ul></li><li>8</li></ul>"
        )

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_lists_ending_abruptly(self):
        """Check that lists ending abruptly are correctly generated"""

        draftjs_data = self.blocks_with_text(
            [
                ["unordered-list-item", 0, "1"],
                ["unordered-list-item", 1, "2"],
                ["unordered-list-item", 2, "3"],
                ["unordered-list-item", 3, "4"],
                ["unstyled", 0, "abc"],
            ]
        )

        expected = "<ul><li>1<ul><li>2<ul><li>3<ul><li>4</li></ul></li></ul></li></ul></li></ul><p>abc</p>"

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_annotation(self):
        """Check that annotations are correctly generated"""

        draftjs_data = {
            "entityMap": {},
            "blocks": [
                {
                    "inlineStyleRanges": [
                        {"length": 5, "style": "ANNOTATION-1", "offset": 6},
                        {"length": 5, "style": "ANNOTATION-2", "offset": 12},
                    ],
                    "data": {
                        "MULTIPLE_HIGHLIGHTS": {
                            "lastHighlightIds": {"ANNOTATION": 2},
                            "highlightsData": {
                                "ANNOTATION-1": {
                                    "data": {
                                        "email": "admin@admin.ro",
                                        "date": "2018-03-30T14:57:53.172Z",
                                        "msg": '{"blocks":[{"key":"ejm11","text":"Annotation 1","type":"unstyled","depth":'
                                        '0,"inlineStyleRanges":[],"entityRanges":[],"data":{}}],"entityMap":{}}',
                                        "author": "admin",
                                        "annotationType": "regular",
                                    },
                                    "type": "ANNOTATION",
                                },
                                "ANNOTATION-2": {
                                    "data": {
                                        "email": "admin@admin.ro",
                                        "date": "2018-03-30T14:58:20.876Z",
                                        "msg": '{"blocks":[{"key":"9i73f","text":"Annotation 2","type":"unstyled","depth":'
                                        '0,"inlineStyleRanges":[],"entityRanges":[],"data":{}},{"key":"d3vb3","text'
                                        '":"Line 2","type":"unstyled","depth":0,"inlineStyleRanges":[],"entityRange'
                                        's":[],"data":{}}],"entityMap":{}}',
                                        "author": "admin",
                                        "annotationType": "regular",
                                    },
                                    "type": "ANNOTATION",
                                },
                            },
                        },
                    },
                    "text": "lorem ipsum dolor",
                    "type": "unstyled",
                    "depth": 0,
                    "key": "2sso6",
                    "entityRanges": [],
                }
            ],
        }
        expected = '<p>lorem <span annotation-id="1">ipsum</span> <span annotation-id="2">dolor</span></p>'

        item = self.build_item(draftjs_data)
        editor = Editor3Content(item)
        html = editor.html
        self.assertEqual(html, expected)

    def test_embed_prepend(self):
        """Check that an embed block can be prepended"""
        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": 'The name of Highlaws comes from the Old English hēah-hlāw, meaning "high mounds".',
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                }
            ],
            "entityMap": {},
        }
        item = self.build_item(draftjs_data)
        body_editor = Editor3Content(item)
        embed_html = '<p class="some_class">some embedded HTML</p>'
        body_editor.prepend("embed", embed_html)
        body_editor.update_item()
        expected = (
            '<div class="embed-block"><p class="some_class">some embedded HTML</p></div><p>The name of Highlaws comes f'
            'rom the Old English hēah-hlāw, meaning "high mounds".</p>'
        )
        self.assertEqual(item["body_html"], expected)

    def _modify_embed_content(self, data):
        data["html"] = data["html"].replace("some embedded HTML", "some modified embed")

    def test_embed_pre_process(self):
        """An embed can be pre-processed with a callback"""
        self.app.config["EMBED_PRE_PROCESS"] = [self._modify_embed_content]
        draftjs_data = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": 'The name of Highlaws comes from the Old English hēah-hlāw, meaning "high mounds".',
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                }
            ],
            "entityMap": {},
        }
        item = self.build_item(draftjs_data)
        body_editor = Editor3Content(item)
        embed_html = '<p class="some_class">some embedded HTML</p>'
        body_editor.prepend("embed", embed_html)
        body_editor.update_item()
        expected = (
            '<div class="embed-block"><p class="some_class">some modified embed</p></div><p>The name of Highlaws comes '
            'from the Old English hēah-hlāw, meaning "high mounds".</p>'
        )
        self.assertEqual(item["body_html"], expected)

    def test_replace_text(self):
        draftjs_data = {
            "blocks": [
                {
                    "key": "foo",
                    "text": "first line text",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [
                        {"offset": 6, "length": 4, "style": "ITALIC"},
                    ],
                    "entityRanges": [],
                },
                {
                    "key": "bar",
                    "text": "second line",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [{"offset": 7, "length": 4, "key": 0}],
                },
            ],
            "entityMap": {
                "0": {
                    "type": "LINK",
                    "mutability": "MUTABLE",
                    "data": {"link": {"href": "http://example.com"}},
                }
            },
        }
        item_editor2 = {"body_html": '<p>first <i>line</i> text</p><p>second <a href="http://example.com">line</a></p>'}
        item_editor3 = self.build_item(draftjs_data, "body_html")

        body_editor = Editor3Content(item_editor3, "body_html")
        body_editor.update_item()
        self.assertEqual(item_editor2["body_html"], item_editor3["body_html"])

        editor_utils.replace_text(item_editor2, "body_html", "first", "initial")
        editor_utils.replace_text(item_editor3, "body_html", "first", "initial")
        self.assertEqual(
            '<p>initial <i>line</i> text</p><p>second <a href="http://example.com">line</a></p>',
            item_editor2["body_html"],
        )
        self.assertEqual(item_editor2["body_html"], item_editor3["body_html"])

        editor_utils.replace_text(item_editor2, "body_html", "text", "foo")
        editor_utils.replace_text(item_editor3, "body_html", "text", "foo")
        self.assertEqual(
            '<p>initial <i>line</i> foo</p><p>second <a href="http://example.com">line</a></p>',
            item_editor2["body_html"],
        )
        self.assertEqual(item_editor2["body_html"], item_editor3["body_html"])

    def test_set_blocks(self):
        draftjs_data = {
            "blocks": [
                {
                    "key": "foo",
                    "text": "first line",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                },
                {
                    "key": "bar",
                    "text": "second line",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                },
            ],
            "entityMap": {},
        }

        item = self.build_item(draftjs_data)
        body_editor = Editor3Content(item)
        body_editor.set_blocks([block for block in body_editor.blocks if block.key == "bar"])
        self.assertEqual(1, len(body_editor.blocks))
        self.assertEqual("bar", body_editor.blocks[0].key)
        self.assertIn("MULTIPLE_HIGHLIGHTS", body_editor.blocks[0].data.get("data"))

        body_editor.set_blocks([])
        self.assertEqual(1, len(body_editor.blocks))
        self.assertIn("MULTIPLE_HIGHLIGHTS", body_editor.blocks[0].data.get("data"))

    def test_replace_text_no_html(self):
        item = {"headline": "foo bar"}
        editor_utils.replace_text(item, "headline", "bar", "baz", is_html=False)
        self.assertEqual("foo baz", item["headline"])

    def test_replace_text_inline_styles(self):
        item = {
            "body_html": "<h1>head</h1>\n<p>lorem <b>this is bold</b> and <b>bold</b> end</p>",
        }
        editor_utils.replace_text(item, "body_html", "bold", "UL")
        self.assertEqual("<h1>head</h1><p>lorem <b>this is UL</b> and <b>UL</b> end</p>", item["body_html"])

    def test_replace_what_you_had_is_what_you_get(self):
        html = "".join(
            [
                "<h1>H1 foo</h1>",
                "<h2>H2 foo</h2>",
                "<h3>H3 foo</h3>",
                "<h4>H4 foo</h4>",
                "<p>P foo</p>",
                "<pre>PRE foo</pre>",
                "<blockquote>BLOCKQUOTE foo</blockquote>",
                '<!-- EMBED START Image {id: "editor_0"} -->\n',
                '<figure><img alt="" src="http://example.com"></figure>\n' '<!-- EMBED END Image {id: "editor_0"} -->',
                "<ul>",
                "<li>LI foo</li>",
                "<li>LI2 foo</li>",
                "</ul>",
                "<ol>",
                "<li>LI OL foo</li>",
                "</ol>",
                "<table>",  # only parsing tables so far, no text replace in it
                "<thead><tr><th><p>th foo</p></th></tr></thead>",
                "<tbody><tr><td><p>td foo</p></td></tr></tbody>",
                "</table>",
                '<p><a href="http://p.com" target="_blank">P</a> foo</p>',
                '<div class="embed-block">',
                "<script>foo</script>",
                "</div>",
                '<div class="embed-block">',
                '<iframe src="http://iframe.com">foo</iframe>',
                "</div>",
            ]
        )

        item = {
            "body_html": html,
            "associations": {
                "editor_0": {
                    "type": "picture",
                    "renditions": {
                        "original": {
                            "href": "http://example.com",
                        },
                    },
                },
            },
        }

        editor_utils.replace_text(item, "body_html", " foo", "")
        self.maxDiff = None
        self.assertEqual(html.replace(" foo", ""), item["body_html"])

        # test we keep draftjs state for next time
        item["body_html"] = "foo"
        editor_utils.replace_text(item, "body_html", " foo", "")
        self.assertEqual(html.replace(" foo", ""), item["body_html"])

    def test_filter_blocks(self):
        item = {
            "body_html": "".join(
                [
                    "<p>first line</p>",
                    "<p>second line</p>",
                ]
            ),
        }

        def block_filter(block):
            return "second" in block.text

        editor_utils.filter_blocks(item, "body_html", block_filter)
        self.assertEqual("<p>second line</p>", item["body_html"])

        item = {"body_html": ""}
        editor_utils.filter_blocks(item, "body_html", block_filter)
        self.assertEqual("", item["body_html"])

    def test_get_content_state_fields(self):
        """Fields with content states are detected correctly"""

        draftjs_data_headline = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": "headline test",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                }
            ],
            "entityMap": {},
        }
        draftjs_data_body_html = {
            "blocks": [
                {
                    "key": "fcbn3",
                    "text": "body_html test",
                    "type": "unstyled",
                    "depth": 0,
                    "inlineStyleRanges": [],
                    "entityRanges": [],
                    "data": {"MULTIPLE_HIGHLIGHTS": {}},
                }
            ],
            "entityMap": {},
        }

        item = self.build_item(draftjs_data_headline, field="headline")
        self.update_item_field(item, draftjs_data_body_html, "body_html")
        found_fields = set(editor_utils.get_content_state_fields(item))
        self.assertEqual(found_fields, {"headline", "body_html"})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.utils import SuperdeskBaseEnum

DICTIONARY_FILE = "file"


class DictionaryType(SuperdeskBaseEnum):
    """Defines Dictionary Type"""

    DICTIONARY = "dictionary"
    ABBREVIATIONS = "abbreviations"


class DictionariesResource(Resource):
    """Dictionaries schema"""

    schema = {
        "name": {"type": "string", "required": True},
        "language_id": {"type": "string", "required": True},
        "content": {
            "type": "dict",
            "schema": {},
            "allow_unknown": True,
        },
        "content_list": {
            "type": "string",
        },
        DICTIONARY_FILE: {
            "type": "file",
        },
        "user": Resource.rel("users", nullable=True),
        "is_active": {
            "type": "string",
            "default": "true",
        },
        "type": {"type": "string", "default": DictionaryType.DICTIONARY.value, "allowed": DictionaryType.values()},
        "_file_id": {"type": "objectid", "nullable": True, "readonly": True},
        "init_version": {"type": "integer"},
    }
    item_methods = ["GET", "PATCH", "PUT", "DELETE"]
    resource_methods = ["GET", "POST", "DELETE"]
    privileges = {"POST": "dictionaries", "PATCH": "dictionaries", "DELETE": "dictionaries"}
    etag_ignore_fields = ["content", "content_list"]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
from superdesk.publish import SUBSCRIBER_TYPES

from superdesk.tests import TestCase
from superdesk.publish.transmitters.file_output import FilePublishService
from superdesk.errors import PublishFileError


class FilePublishServiceTest(TestCase):
    def setUp(self):
        self.fixtures = os.path.join(os.path.abspath(os.path.dirname(__file__)))
        self.subscribers = [
            {
                "_id": "1",
                "name": "Test",
                "media_type": "media",
                "subscriber_type": SUBSCRIBER_TYPES.WIRE,
                "is_active": True,
                "sequence_num_settings": {"max": 10, "min": 1},
                "destinations": [
                    {"name": "test", "delivery_type": "File", "format": "nitf", "config": {"file_path": self.fixtures}}
                ],
            }
        ]

    def test_file_write(self):
        item = {
            "item_id": "test_file_name",
            "item_version": 1,
            "published_seq_num": 1,
            "formatted_item": "I was here",
            "encoded_item": b"I was here",
            "item_encoding": "utf-8",
            "destination": {
                "name": "test",
                "delivery_type": "File",
                "format": "nitf",
                "config": {"file_path": self.fixtures, "file_extension": "txt"},
            },
        }
        service = FilePublishService()
        try:
            service._transmit(item, self.subscribers)
            self.assertTrue(True)
        finally:
            path = os.path.join(self.fixtures, "test_file_name-1-1.txt")
            if os.path.isfile(path):
                os.remove(path)

    def test_format_default_file_extension(self):
        item = {
            "item_id": "test_file_name",
            "item_version": 1,
            "published_seq_num": 1,
            "formatted_item": "I was here",
            "encoded_item": b"I was here",
            "item_encoding": "utf-8",
            "destination": {
                "name": "test",
                "delivery_type": "File",
                "format": "nitf",
                "config": {"file_path": self.fixtures},
            },
        }
        service = FilePublishService()
        try:
            service._transmit(item, self.subscribers)
            self.assertTrue(True)
        finally:
            path = os.path.join(self.fixtures, "test_file_name-1-1.ntf")
            if os.path.isfile(path):
                os.remove(path)

        item["destination"]["config"]["file_extension"] = ""
        try:
            service._transmit(item, self.subscribers)
            self.assertTrue(True)
        finally:
            path = os.path.join(self.fixtures, "test_file_name-1-1.ntf")
            if os.path.isfile(path):
                os.remove(path)

    def test_file_write_fail(self):
        self.fixtures = os.path.join(os.path.abspath(os.path.dirname(__file__) + "/xyz"))
        item = {
            "item_id": "test_file_name",
            "item_version": 1,
            "formatted_item": "I was here",
            "encoded_item": b"I was here",
            "item_encoding": "utf-8",
            "destination": {
                "name": "test",
                "delivery_type": "File",
                "format": "nitf",
                "config": {"file_path": self.fixtures},
            },
        }

        with self.app.app_context():
            service = FilePublishService()
            try:
                service._transmit(item, self.subscribers)
            except PublishFileError as ex:
                self.assertEqual(str(ex), "PublishFileError Error 13000 - File publish error")
                self.assertEqual(ex.code, 13000)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.common.models.base_model import BaseModel
from apps.item_lock.models.item import ItemValidator


class ItemAutosaveModel(BaseModel):
    def __init__(self, data_layer):
        BaseModel.__init__(self, "archive_autosave", data_layer, {}, ItemValidator())

    @classmethod
    def name(cls):
        return "item_autosave"



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""
A module containing configuration of the Superdesk's production API.

The meaning of configuration options is described in the Eve framework
`documentation <http://python-eve.org/config.html#global-configuration>`_.
"""

from superdesk.default_settings import env, urlparse

from superdesk.default_settings import (  # noqa
    DEBUG,
    SUPERDESK_TESTING,
    MONGO_URI,
    ELASTICSEARCH_INDEX,
    ELASTICSEARCH_URL,
    AMAZON_ACCESS_KEY_ID,
    AMAZON_SECRET_ACCESS_KEY,
    AMAZON_REGION,
    AMAZON_CONTAINER_NAME,
    AMAZON_S3_SUBFOLDER,
    AMAZON_OBJECT_ACL,
    AMAZON_ENDPOINT_URL,
    strtobool,
)

SECRET_KEY = env("PRODAPI_SECRET_KEY", "")

PRODAPI_INSTALLED_APPS = (
    "prod_api.items",
    "prod_api.assets",
    "prod_api.desks",
    "prod_api.planning",
    "prod_api.contacts",
    "prod_api.users",
)

# NOTE: no trailing slash for the PRODAPI_URL setting!
PRODAPI_URL = env("PRODAPI_URL", "http://localhost:5500")
URL_PREFIX = env("PRODAPI_URL_PREFIX", "prodapi")
API_VERSION = "v1"
MEDIA_PREFIX = env("MEDIA_PREFIX", "{}/{}/{}/assets".format(PRODAPI_URL.rstrip("/"), URL_PREFIX, API_VERSION))

# date formats
DATE_FORMAT = "%Y-%m-%dT%H:%M:%S+0000"
ELASTIC_DATE_FORMAT = "%Y-%m-%d"

# response in json
XML = False

# authorisation server
AUTH_SERVER_SHARED_SECRET = env("AUTH_SERVER_SHARED_SECRET", "")

# authentication
PRODAPI_AUTH_ENABLED = strtobool(env("PRODAPI_AUTH_ENABLED", "true"))



import asyncio
import unittest
from json import dumps
from datetime import datetime, timedelta
from superdesk.websockets_comms import SocketCommunication


class TestClient:
    def __init__(self):
        self.open = True
        self.messages = []

    def send(self, message):
        self.messages.append(message)


class WebsocketsTestCase(unittest.TestCase):
    def test_broadcast(self):
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        client = TestClient()
        com = SocketCommunication("host", "port", "url")
        com.clients.add(client)

        loop.run_until_complete(
            com.broadcast(dumps({"event": "ingest:update", "_created": datetime.now().isoformat()}))
        )
        self.assertEqual(1, len(client.messages))

        loop.run_until_complete(
            com.broadcast(dumps({"event": "ingest:update", "_created": datetime.now().isoformat()}))
        )
        self.assertEqual(1, len(client.messages))

        loop.run_until_complete(com.broadcast(dumps({"event": "foo", "_created": datetime.now().isoformat()})))
        self.assertEqual(2, len(client.messages))

        loop.run_until_complete(com.broadcast(dumps({"event": "foo", "_created": datetime.now().isoformat()})))
        self.assertEqual(3, len(client.messages))

        loop.run_until_complete(
            com.broadcast(
                dumps({"event": "ingest:update", "_created": (datetime.now() + timedelta(seconds=3600)).isoformat()})
            )
        )
        self.assertEqual(4, len(client.messages))



import unittest
from unittest.mock import patch
from pathlib import Path

from superdesk.backend_meta import backend_meta


class BackendMetaTestCase(unittest.TestCase):
    @patch.object(backend_meta, "config", object)
    def test_get_commit_href(self):
        href = backend_meta.BackendMetaService.get_commit_href("superdesk", "xyz")
        self.assertEqual(href, "https://github.com/superdesk/superdesk/commit/xyz")

    @patch.object(backend_meta.config, "REPO_OVERRIDE", {"superdesk": "superdesk-test"}, create=True)
    def test_get_commit_href_override(self):
        href = backend_meta.BackendMetaService.get_commit_href("superdesk", "xyz")
        self.assertEqual(href, "https://github.com/superdesk/superdesk-test/commit/xyz")

    @patch.object(backend_meta, "pkg_version")
    def test_get_package_version_dev(self, pkg_version):
        pkg_version.return_value = "2.0.123.dev123+g01abcde"
        version_data = backend_meta.BackendMetaService.get_package_version("superdesk-core")
        self.assertEqual(
            version_data,
            {
                "name": "superdesk-core",
                "version": "2.0.123.dev123+g01abcde",
                "semver": "2.0.123",
                "revision": "01abcde",
                "href": "https://github.com/superdesk/superdesk-core/commit/01abcde",
            },
        )

    @patch.object(backend_meta, "pkg_version")
    def test_get_package_version_stable(self, pkg_version):
        pkg_version.return_value = "2.0.123"
        version_data = backend_meta.BackendMetaService.get_package_version("superdesk-core")
        self.assertEqual(
            version_data,
            {
                "name": "superdesk-core",
                "version": "2.0.123",
                "semver": "2.0.123",
                "href": "https://pypi.org/project/superdesk-core/2.0.123/",
            },
        )

    @patch.object(backend_meta.BackendMetaService, "find_dir")
    def test_get_nodemod_version_rev(self, find_dir):
        fixtures_path = Path(__file__).parent / "fixtures" / "backend_meta" / "dev"
        find_dir.return_value = fixtures_path
        backend_meta_service = backend_meta.BackendMetaService()
        version_data = backend_meta_service.get_nodemod_version("superdesk-core", repo="superdesk-client-core")
        self.assertEqual(
            version_data,
            {
                "name": "superdesk-client-core",
                "version": "141474f6643473dee1c6794989e9b231daf13465",
                "revision": "141474f6643473dee1c6794989e9b231daf13465",
                "href": "https://github.com/superdesk/superdesk-client-core/commit/141474f6643473dee1c6794989e9b231daf13465",
            },
        )

    @patch.object(backend_meta.BackendMetaService, "find_dir")
    def test_get_nodemod_version_stable(self, find_dir):
        fixtures_path = Path(__file__).parent / "fixtures" / "backend_meta" / "stable"
        find_dir.return_value = fixtures_path
        backend_meta_service = backend_meta.BackendMetaService()
        version_data = backend_meta_service.get_nodemod_version("superdesk-core", repo="superdesk-client-core")
        self.assertEqual(
            version_data,
            {
                "name": "superdesk-client-core",
                "version": "2.1.0",
                "semver": "2.1.0",
                "href": "https://www.npmjs.com/package/superdesk-core/v/2.1.0",
            },
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import get_resource_service
from superdesk.metadata.item import CONTENT_STATE
from eve.utils import config
from apps.publish.enqueue.enqueue_service import EnqueueService


class EnqueueCorrectedService(EnqueueService):

    publish_type = "correct"
    published_state = "corrected"

    def get_subscribers(self, doc, target_media_type):
        """Get the subscribers for this document based on the target_media_type for article Correction.

        1. The article is sent to Subscribers (digital and wire) who has received the article previously.
        2. Fetch Active Subscribers. After fetching exclude those who received the article previously from
           active subscribers list.
        3. If article has 'targeted_for' property then exclude subscribers of type Internet from Subscribers list.
        4. Filter the subscriber that have not received the article previously against publish filters
        and global filters for this document.

        :param doc: Document to correct
        :param target_media_type: Valid values are - Wire, Digital.
        :return: (list, dict, dict) List of filtered subscribers, product codes per subscriber,
                associations per subscriber
        """
        subscribers, subscribers_yet_to_receive = [], []

        # step 1
        query = {
            "$and": [
                {"item_id": doc["item_id"]},
                {"publishing_action": {"$in": [CONTENT_STATE.PUBLISHED, CONTENT_STATE.CORRECTED]}},
            ]
        }

        subscribers, subscriber_codes, previous_associations = self._get_subscribers_for_previously_sent_items(query)

        if subscribers:
            # Step 2
            query = {"is_active": True}
            active_subscribers = list(get_resource_service("subscribers").get(req=None, lookup=query))
            subscribers_yet_to_receive = [
                a for a in active_subscribers if not any(a[config.ID_FIELD] == s[config.ID_FIELD] for s in subscribers)
            ]

            if len(subscribers_yet_to_receive) > 0:
                # Step 3
                if doc.get("target_regions"):
                    subscribers_yet_to_receive = list(self.non_digital(subscribers_yet_to_receive))
                # Step 4
                subscribers_yet_to_receive, codes = self.filter_subscribers(
                    doc, subscribers_yet_to_receive, target_media_type
                )
                if codes:
                    subscriber_codes.update(codes)

        subscribers = subscribers + subscribers_yet_to_receive
        associations = self._filter_subscribers_for_associations(
            subscribers, doc, target_media_type, previous_associations
        )

        return subscribers, subscriber_codes, associations



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : Gyan
# Creation: 2018-12-27 16:03

from superdesk.commands.data_updates import BaseDataUpdate
from eve.utils import config


# This script replaces the whole json of related item with respective _id only
class DataUpdate(BaseDataUpdate):

    resource = "archive"

    def forwards(self, mongodb_collection, mongodb_database):
        # To find the related content from vocabularies
        related_content = list(mongodb_database["vocabularies"].find({"field_type": "related_content"}))

        collection = mongodb_database["archive"]

        for item in collection.find({"associations": {"$ne": None}}):
            for item_name, item_obj in item["associations"].items():
                if item_obj and related_content:
                    if item_name.split("--")[0] in [content["_id"] for content in related_content]:
                        related_item_id = item_obj[config.ID_FIELD]

                        updates = {"$set": {}}
                        updates["$set"]["associations." + item_name] = {"_id": related_item_id}

                        print(collection.update({"_id": item["_id"]}, updates))

    def backwards(self, mongodb_collection, mongodb_database):
        raise NotImplementedError()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from .anpa import ANPAFeedParser
from superdesk.io.registry import register_feed_parser
from superdesk.io.iptc import subject_codes
from flask import current_app as app
from apps.archive.common import format_dateline_to_locmmmddsrc
from superdesk.utc import get_date
from superdesk import get_resource_service
from superdesk.etree import parse_html
import logging
import re

logger = logging.getLogger("AP_ANPAFeedParser")


class AP_ANPAFeedParser(ANPAFeedParser):
    """
    Feed parser for AP supplied ANPA, maps category codes and maps the prefix on some sluglines to subject codes
    """

    NAME = "ap_anpa1312"

    label = "AP ANPA Parser"

    slug_code_map = {  # Basketball
        "BKC-": "15008000",
        "BKL-": "15008000",
        "BKN-": "15008000",
        "BKW-": "15008000",
        "BKO-": "15008000",
        # Baseball
        "BBA-": "15007000",
        "BBC-": "15007000",
        "BBM-": "15007000",
        "BBN-": "15007000",
        "BBO-": "15007000",
        "BBY-": "15007000",
        # Boxing
        "BOX-": "15014000",
        # Motor racing
        "CAR-": "15039000",
        # Cycling
        "CYC-": "15019000",
        # American Football
        "FBL-": "15003000",
        "FBN-": "15003000",
        "FBO-": "15003000",
        # Figure skating
        "FIG-": "15025000",
        # Golf
        "GLF-": "15027000",
        # Ice Hockey
        "HKN-": "15031000",
        "HKO-": "15031000",
        # Horse Racing
        "RAC-": "15030000",
        # Soccer
        "SOC-": "15054000",
        # Tennis
        "TEN-": "15065000",
        # Cricket
        "CRI-": "15017000",
        # Rugby league
        "RGL-": "15048000",
    }

    def parse(self, file_path, provider=None):
        item = super().parse(file_path, provider)
        self.ap_derive_dateline(item)
        self.map_category_codes(item)
        self.map_sluglines_to_subjects(item)
        if "headline" not in item:
            self._fix_headline(item)
        return item

    def map_category_codes(self, item):
        """Map the category code that has been received to a more palatable value

        :param item:
        :return:
        """
        category_code_map = get_resource_service("vocabularies").find_one(req=None, _id="ap_category_map")
        if category_code_map:
            map = {c["ap_code"]: c["category_code"] for c in category_code_map["items"] if c["is_active"]}
            for category in item.get("anpa_category", []):
                category["qcode"] = map.get(category.get("qcode").lower(), map.get("default"))

    def map_sluglines_to_subjects(self, item):
        """The first few characters of the slugline may match AP supplimetal categories this is used to set the subject code.

        :param item:
        :return:
        """
        if len(item.get("slugline", "")) > 4:
            qcode = self.slug_code_map.get(item["slugline"][:4])
            if qcode:
                try:
                    item["subject"] = []
                    item["subject"].append({"qcode": qcode, "name": subject_codes[qcode]})
                except KeyError:
                    logger.debug("Subject code '%s' not found" % qcode)

    def ap_derive_dateline(self, item):
        """This function looks for a dateline in the article body an uses that.

        :param item:
        :return: item populated with a dateline
        """
        try:
            html = item.get("body_html")
            if html:
                parsed = parse_html(html, content="html")
                for par in parsed.xpath("/div/child::*"):
                    if not par.text:
                        continue
                    city, source, the_rest = par.text.partition(" (AP) _ ")
                    if source:
                        # sometimes the city is followed by a comma and either a date or a state
                        city = city.split(",")[0]
                        if any(char.isdigit() for char in city):
                            return
                        cities = app.locators.find_cities()
                        located = [c for c in cities if c["city"].lower() == city.lower()]
                        item.setdefault("dateline", {})
                        item["dateline"]["located"] = (
                            located[0]
                            if len(located) == 1
                            else {"city_code": city, "city": city, "tz": "UTC", "dateline": "city"}
                        )
                        item["dateline"]["source"] = item.get("original_source", "AP")
                        item["dateline"]["text"] = format_dateline_to_locmmmddsrc(
                            item["dateline"]["located"],
                            get_date(item["firstcreated"]),
                            source=item.get("original_source", "AP"),
                        )
                        break

            return item
        except Exception:
            logging.exception("AP dateline extraction exception")

    def _parse_ednote(self, header_lines, item):
        """
        Attempt to parse the ednote, If no ednote is found by the base class try another pattern
        :param header_lines:
        :param item:
        :return:
        """
        super()._parse_ednote(header_lines, item)
        if not item.get("ednote"):
            for line in header_lines:
                m = re.search("(\(?)Eds: (.*)(\)?)", line)
                if m:
                    item["ednote"] = m.group(0).strip()[:-1].replace("Eds: ", "")
                    break

    def _fix_headline(self, item):
        """
        AP Alerts do not get a healdine parsed out so pick up the first par of the content and put it in the headline
        :param item:
        :return:
        """
        try:
            html = item.get("body_html")
            if html:
                parsed = parse_html(html, content="html")
                pars = parsed.xpath("/div/child::*")
                if pars and len(pars) > 0:
                    city, source, the_rest = pars[0].text.partition(" (AP) _ ")
                    if the_rest:
                        item["headline"] = the_rest
                    else:
                        item["headline"] = pars[0].text
        except Exception:
            pass


register_feed_parser(AP_ANPAFeedParser.NAME, AP_ANPAFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import hashlib
import logging
import email.policy

from datetime import timedelta
from superdesk.utc import utcnow
from superdesk.lock import lock, unlock
from bson.json_util import dumps
from flask_mail import Message
from superdesk.celery_app import celery
from flask import current_app as app, render_template, render_template_string
from superdesk import get_resource_service

logger = logging.getLogger(__name__)


class SuperdeskMessage(Message):
    def as_bytes(self):
        msg = self._message()
        return msg.as_bytes(policy=email.policy.HTTP)


EMAIL_TIMESTAMP_RESOURCE = "email_timestamps"


@celery.task(bind=True, max_retries=3, soft_time_limit=120)
def send_email(self, subject, sender, recipients, text_body, html_body, cc=None, bcc=None, attachments=None):
    _id = get_activity_digest(
        {
            "subject": subject,
            "recipients": recipients,
            "text_body": text_body,
            "html_body": html_body,
            "cc": cc,
            "bcc": bcc,
        }
    )

    lock_id = "email:%s" % _id
    if not lock(lock_id, expire=120):
        return

    try:
        subject_line = subject.split("\n")[0]
    except AttributeError:
        subject_line = None

    try:
        msg = SuperdeskMessage(
            subject_line,
            sender=sender,
            recipients=recipients,
            cc=cc,
            bcc=bcc,
            body=text_body,
            html=html_body,
            attachments=attachments,
        )
        return app.mail.send(msg)
    except OSError:
        logger.exception("can not send email %s", subject)
    finally:
        unlock(lock_id, remove=True)


def send_activate_account_email(doc, activate_ttl):
    user = get_resource_service("users").find_one(req=None, _id=doc["user"])
    first_name = user.get("first_name")
    app_name = app.config["APPLICATION_NAME"]
    admins = app.config["ADMINS"]
    client_url = app.config["CLIENT_URL"].rstrip("/")
    url = "{}/#/reset-password?token={}".format(client_url, doc["token"])
    hours = activate_ttl * 24
    subject = render_template("account_created_subject.txt", app_name=app_name)
    text_body = render_template(
        "account_created.txt",
        app_name=app_name,
        user=user,
        first_name=first_name,
        instance_url=client_url,
        expires=hours,
        url=url,
    )
    html_body = render_template(
        "account_created.html",
        app_name=app_name,
        user=user,
        first_name=first_name,
        instance_url=client_url,
        expires=hours,
        url=url,
    )
    send_email.delay(
        subject=subject, sender=admins[0], recipients=[doc["email"]], text_body=text_body, html_body=html_body
    )


def send_user_status_changed_email(recipients, status):
    admins = app.config["ADMINS"]
    app_name = app.config["APPLICATION_NAME"]
    subject = render_template("account_status_changed_subject.txt", app_name=app_name, status=status)
    text_body = render_template("account_status_changed.txt", app_name=app_name, status=status)
    html_body = render_template("account_status_changed.html", app_name=app_name, status=status)
    send_email.delay(subject=subject, sender=admins[0], recipients=recipients, text_body=text_body, html_body=html_body)


def send_reset_password_email(doc, token_ttl):
    admins = app.config["ADMINS"]
    client_url = app.config["CLIENT_URL"].rstrip("/")
    app_name = app.config["APPLICATION_NAME"]
    url = "{}/#/reset-password?token={}".format(client_url, doc["token"])
    hours = token_ttl * 24
    subject = render_template("reset_password_subject.txt")
    text_body = render_template("reset_password.txt", email=doc["email"], expires=hours, url=url, app_name=app_name)
    html_body = render_template("reset_password.html", email=doc["email"], expires=hours, url=url, app_name=app_name)
    send_email.delay(
        subject=subject, sender=admins[0], recipients=[doc["email"]], text_body=text_body, html_body=html_body
    )


def send_user_mentioned_email(recipients, user_name, doc, url):
    logging.info("sending mention email to: %s", recipients)
    admins = app.config["ADMINS"]
    app_name = app.config["APPLICATION_NAME"]
    subject = render_template("user_mention_subject.txt", username=user_name)
    text_body = render_template("user_mention.txt", text=doc["text"], username=user_name, link=url, app_name=app_name)
    html_body = render_template("user_mention.html", text=doc["text"], username=user_name, link=url, app_name=app_name)
    send_email.delay(subject=subject, sender=admins[0], recipients=recipients, text_body=text_body, html_body=html_body)


def get_activity_digest(value):
    h = hashlib.sha1()
    json_encoder = app.data.json_encoder_class()
    h.update(dumps(value, sort_keys=True, default=json_encoder.default).encode("utf-8"))
    return h.hexdigest()


def send_activity_emails(activity, recipients):
    now = utcnow()
    message_id = get_activity_digest(activity)
    # there is no resource for email timestamps registered,
    # so use users resoure to get pymongo db
    email_timestamps = app.data.mongo.pymongo("users").db[EMAIL_TIMESTAMP_RESOURCE]
    last_message_info = email_timestamps.find_one(message_id)
    resend_interval = app.config.get("EMAIL_NOTIFICATION_RESEND", 24)

    if last_message_info and last_message_info["_created"] + timedelta(hours=resend_interval) > now:
        return

    admins = app.config["ADMINS"]
    app_name = app.config["APPLICATION_NAME"]
    link = activity.get("data", {}).get("link", None)

    notification = render_template_string(activity.get("message"), **activity.get("data"))
    text_body = render_template("notification.txt", notification=notification, app_name=app_name, link=link)
    html_body = render_template("notification.html", notification=notification, app_name=app_name, link=link)
    subject = render_template("notification_subject.txt", notification=notification)

    send_email.delay(subject=subject, sender=admins[0], recipients=recipients, text_body=text_body, html_body=html_body)
    email_timestamps.update({"_id": message_id}, {"_id": message_id, "_created": now}, upsert=True)


def send_article_killed_email(article, recipients, transmitted_at):
    admins = app.config["ADMINS"]
    app_name = app.config["APPLICATION_NAME"]
    place = next(iter(article.get("place") or []), "")
    if place:
        place = place.get("qcode", "")
    body = article.get("body_html", "")

    subject = article.get("headline", "Kill Notification")
    operation = article.get("operation")

    text_body = render_template("article_killed.txt", app_name=app_name, place=place, body=body, operation=operation)
    html_body = render_template("article_killed.html", app_name=app_name, place=place, body=body, operation=operation)

    send_email.delay(subject=subject, sender=admins[0], recipients=recipients, text_body=text_body, html_body=html_body)


def send_translation_changed(username, article, recipients):
    admins = app.config["ADMINS"]
    app_name = app.config["APPLICATION_NAME"]
    client_url = app.config.get("CLIENT_URL", "").rstrip("/")

    link = "{}/#/workspace?item={}&action=edit".format(client_url, article["guid"])
    headline = article.get("headline", link)

    subject = render_template("translation_changed_subject.txt", headline=headline)
    text_body = render_template(
        "translation_changed.txt", app_name=app_name, username=username, link=link, headline=headline
    )
    html_body = render_template(
        "translation_changed.html", app_name=app_name, username=username, link=link, headline=headline
    )

    send_email.delay(subject=subject, sender=admins[0], recipients=recipients, text_body=text_body, html_body=html_body)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
import logging
import requests
from superdesk.errors import SuperdeskApiError
from superdesk.text_checkers.spellcheckers import CAP_SPELLING
from superdesk.text_checkers.spellcheckers.base import SpellcheckerBase

logger = logging.getLogger(__name__)
OPT_API_KEY = "LEUVEN_DUTCH_API_KEY"
API_URL = "https://dev.schrijfhulp.be/v1/{method}/"
START_MARKER = "##__ERR_START__##"
END_MARKER = "##__ERR_END__##"


class LeuvenDutch(SpellcheckerBase):
    """University of Leuven Dutch grammar/spellchecker integration

    The LEUVEN_DUTCH_API_KEY setting (or environment variable) must be set to the API key
    """

    name = "leuven_dutch"
    label = "University of Leuven Dutch spellchecker"
    capacities = (CAP_SPELLING,)
    languages = ["nl"]

    def __init__(self, app):
        super().__init__(app)
        self.api_key = self.config.get(OPT_API_KEY, os.environ.get(OPT_API_KEY))

    def check(self, text, language=None):
        check_url = API_URL.format(method="spellingchecker")
        data = {
            "key": self.api_key,
            "input": text,
            "startmarker": START_MARKER,
            "endmarker": END_MARKER,
        }
        r = requests.post(check_url, data=data, timeout=self.CHECK_TIMEOUT)
        if r.status_code != 200:
            raise SuperdeskApiError.internalError("Unexpected return code from {}".format(self.name))

        data = r.json()

        err_list = []
        check_data = {"errors": err_list}
        len_end_marker = len(END_MARKER)
        output = data["spellingchecker"]["output"]
        marked = output["marked"].split(START_MARKER)
        # the first item in "marked" is unmarked text, we start our index there
        text_idx = len(marked.pop(0))

        for marked_part in marked:
            mistake = marked_part[: marked_part.find(END_MARKER)]
            ercorr_data = {
                "startOffset": text_idx,
                "text": mistake,
                "type": "spelling",
            }
            err_list.append(ercorr_data)
            text_idx += len(marked_part) - len_end_marker

        return check_data

    def suggest(self, text, language=None):
        check_url = API_URL.format(method="suggesties")
        data = {
            "key": self.api_key,
            "input": text,
        }
        r = requests.post(check_url, data=data, timeout=self.SUGGEST_TIMEOUT)
        if r.status_code != 200:
            raise SuperdeskApiError.internalError("Unexpected return code from {}".format(self.name))
        return {"suggestions": self.list2suggestions(r.json()["suggesties"]["output"].get("suggesties", []))}

    def available(self):
        if not self.api_key:
            logger.warning(
                "API key is not set for {label}, please set {opt} variable to use it".format(
                    label=self.label, opt=OPT_API_KEY
                )
            )
            return False
        return True



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import flask
import logging
import superdesk

from superdesk.metadata.item import ITEM_STATE, CONTENT_STATE, ITEM_TYPE, CONTENT_TYPE, PUBLISH_STATES
from superdesk.errors import SuperdeskApiError
from superdesk.notification import push_notification
from superdesk.users.services import current_user_has_privilege
from superdesk.utc import utcnow
from superdesk.lock import lock, unlock
from eve.utils import config
from eve.versioning import resolve_document_version, insert_versioning_documents

from apps.common.components.base_component import BaseComponent
from apps.common.models.utils import get_model
from apps.content import push_content_notification
from apps.packages.package_service import PackageService
from ..models.item import ItemModel
from flask_babel import _

LOCK_USER = "lock_user"
LOCK_SESSION = "lock_session"
LOCK_TIME = "lock_time"
LOCK_ACTION = "lock_action"
STATUS = "_status"
TASK = "task"
logger = logging.getLogger(__name__)


def push_unlock_notification(item, user_id, session_id):
    push_notification(
        "item:unlock",
        item=str(item.get(config.ID_FIELD)),
        item_version=str(item.get(config.VERSION)),
        state=item.get(ITEM_STATE),
        user=str(user_id),
        lock_session=str(session_id),
        _etag=item.get(config.ETAG),
    )


class ItemLock(BaseComponent):
    def __init__(self, app):
        self.app = app
        self.app.on_session_end += self.on_session_end

    @classmethod
    def name(cls):
        return "item_lock"

    def lock(self, item_filter, user_id, session_id, action):
        item_model = get_model(ItemModel)
        item = item_model.find_one(item_filter)

        # set the lock_id it per item
        lock_id = "item_lock {}".format(item.get(config.ID_FIELD))

        if not item:
            raise SuperdeskApiError.notFoundError()

        # get the lock it not raise forbidden exception
        if not lock(lock_id, expire=5):
            raise SuperdeskApiError.forbiddenError(message=_("Item is locked by another user."))

        try:
            can_user_lock, error_message = self.can_lock(item, user_id, session_id)

            if can_user_lock:
                self.app.on_item_lock(item, user_id)
                updates = {LOCK_USER: user_id, LOCK_SESSION: session_id, LOCK_TIME: utcnow()}
                if action:
                    updates[LOCK_ACTION] = action

                updates[TASK] = item.get(TASK)
                if updates.get(TASK):
                    updates[TASK]["user"] = user_id
                else:
                    updates[TASK] = {"user": user_id}

                # tasks service will update the user
                superdesk.get_resource_service("tasks").assign_user(item[config.ID_FIELD], updates)

                item = item_model.find_one(item_filter)
                self.app.on_item_locked(item, user_id)
                push_notification(
                    "item:lock",
                    item=str(item.get(config.ID_FIELD)),
                    item_version=str(item.get(config.VERSION)),
                    user=str(user_id),
                    lock_time=updates[LOCK_TIME],
                    lock_session=str(session_id),
                    _etag=item.get(config.ETAG),
                )
            else:
                raise SuperdeskApiError.forbiddenError(message=error_message)

            item = item_model.find_one(item_filter)
            return item
        finally:
            # unlock the lock :)
            unlock(lock_id, remove=True)

    def unlock(self, item_filter, user_id, session_id, etag):
        item_model = get_model(ItemModel)
        item = item_model.find_one(item_filter)

        if not item:
            raise SuperdeskApiError.notFoundError()

        if not item.get(LOCK_USER):
            raise SuperdeskApiError.badRequestError(message=_("Item is not locked."))

        can_user_unlock, error_message = self.can_unlock(item, user_id)

        if can_user_unlock:
            self.app.on_item_unlock(item, user_id)
            updates = {}

            # delete the item if nothing is saved so far
            # version 0 created on lock item
            if item.get(config.VERSION, 0) == 0 and item[ITEM_STATE] == CONTENT_STATE.DRAFT:
                if item.get(ITEM_TYPE) == CONTENT_TYPE.COMPOSITE:
                    # if item is composite then update referenced items in package.
                    PackageService().update_groups({}, item)

                superdesk.get_resource_service("archive").delete_action(lookup={"_id": item["_id"]})
                push_content_notification([item])
            else:
                updates = {
                    LOCK_USER: None,
                    LOCK_SESSION: None,
                    LOCK_TIME: None,
                    LOCK_ACTION: None,
                    "force_unlock": True,
                }
                autosave = superdesk.get_resource_service("archive_autosave").find_one(req=None, _id=item["_id"])
                if autosave and item[ITEM_STATE] not in PUBLISH_STATES:
                    if not hasattr(flask.g, "user"):  # user is not set when session expires
                        flask.g.user = superdesk.get_resource_service("users").find_one(req=None, _id=user_id)
                    autosave.update(updates)
                    resolve_document_version(autosave, "archive", "PATCH", item)
                    superdesk.get_resource_service("archive").patch(item["_id"], autosave)
                    item = superdesk.get_resource_service("archive").find_one(req=None, _id=item["_id"])
                    insert_versioning_documents("archive", item)
                else:
                    item_model.update(item_filter, updates)
                    item = item_model.find_one(item_filter)
                self.app.on_item_unlocked(item, user_id)

            push_unlock_notification(item, user_id, session_id)
        else:
            raise SuperdeskApiError.forbiddenError(message=error_message)

        return item

    def unlock_session(self, user_id, session_id):
        item_model = get_model(ItemModel)
        items = item_model.find({LOCK_SESSION: str(session_id)})

        for item in items:
            self.unlock({"_id": item["_id"]}, user_id, session_id, None)

    def can_lock(self, item, user_id, session_id):
        """
        Function checks whether user can lock the item or not. If not then raises exception.
        """
        can_user_edit, error_message = superdesk.get_resource_service("archive").can_edit(item, user_id)

        if can_user_edit:
            if item.get(LOCK_USER):
                if str(item.get(LOCK_USER, "")) == str(user_id) and str(item.get(LOCK_SESSION)) != str(session_id):
                    return False, "Item is locked by you in another session."
                else:
                    if str(item.get(LOCK_USER, "")) != str(user_id):
                        return False, "Item is locked by another user."
        else:
            return False, error_message

        return True, ""

    def can_unlock(self, item, user_id):
        """
        Function checks whether user can unlock the item or not.
        """
        can_user_edit, error_message = superdesk.get_resource_service("archive").can_edit(item, user_id)

        if can_user_edit:
            if not (
                str(item.get(LOCK_USER, "")) == str(user_id)
                or (current_user_has_privilege("archive") and current_user_has_privilege("unlock"))
            ):
                return False, "You don't have permissions to unlock an item."
        else:
            return False, error_message

        return True, ""

    def on_session_end(self, user_id, session_id):
        self.unlock_session(user_id, session_id)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource, build_custom_hateoas
from superdesk.services import BaseService
from .common import CUSTOM_HATEOAS
from superdesk.metadata.utils import aggregations
from .archive import ArchiveResource
import superdesk


class UserContentResource(Resource):
    endpoint_name = "user_content"
    item_url = ArchiveResource.item_url
    url = 'users/<regex("[a-f0-9]{24}"):original_creator>/content'
    schema = ArchiveResource.schema
    datasource = {
        "source": "archive",
        "aggregations": aggregations,
        "elastic_filter": {
            "and": [
                {"not": {"exists": {"field": "task.desk"}}},
                {"not": {"term": {"version": 0}}},
            ]
        },
    }
    resource_methods = ["GET", "POST"]
    item_methods = ["GET", "PATCH", "DELETE"]
    resource_title = endpoint_name


class UserContentService(BaseService):
    def on_fetched(self, docs):
        """
        Overriding this to handle existing data in Mongo & Elastic
        """
        self.enhance_items(docs["_items"])

    def on_fetched_item(self, doc):
        self.enhance_items([doc])

    def enhance_items(self, items):
        for item in items:
            build_custom_hateoas(CUSTOM_HATEOAS, item)


superdesk.workflow_state("draft")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.archive.archive import ArchiveResource, SOURCE as ARCHIVE
from superdesk.metadata.utils import item_url
import logging
from functools import partial
from flask import request, current_app as app
from superdesk import get_resource_service, Service, config
from superdesk.errors import SuperdeskApiError
from superdesk.metadata.item import CONTENT_TYPE, ITEM_TYPE, ITEM_STATE, CONTENT_STATE
from superdesk.publish import SUBSCRIBER_TYPES
from apps.archive.common import is_genre, BROADCAST_GENRE, ITEM_RESEND
from apps.publish.enqueue import get_enqueue_service
from apps.archive.common import ITEM_OPERATION
from flask_babel import _

logger = logging.getLogger(__name__)


class ResendResource(ArchiveResource):
    endpoint_name = "archive_resend"
    resource_title = endpoint_name

    schema = {"subscribers": {"type": "list"}, "version": {"type": "integer"}}

    url = "archive/<{0}:original_id>/resend".format(item_url)
    resource_methods = ["POST"]
    privileges = {"POST": "resend"}


class ResendService(Service):

    digital = partial(
        filter, lambda s: (s.get("subscriber_type", "") in {SUBSCRIBER_TYPES.DIGITAL, SUBSCRIBER_TYPES.ALL})
    )

    def create(self, docs, **kwargs):
        doc = docs[0] if len(docs) > 0 else {}
        article_id = request.view_args["original_id"]
        article_version = doc.get("version")
        article = self._validate_article(article_id, article_version)
        subscribers = self._validate_subscribers(doc.get("subscribers"), article)
        get_enqueue_service(article.get(ITEM_OPERATION)).resend(article, subscribers)
        app.on_archive_item_updated({"subscribers": doc.get("subscribers")}, article, ITEM_RESEND)
        return [article_id]

    def _validate_subscribers(self, subscriber_ids, article):
        if not subscriber_ids:
            raise SuperdeskApiError.badRequestError(message=_("No subscribers selected!"))

        query = {"$and": [{config.ID_FIELD: {"$in": list(subscriber_ids)}}, {"is_active": True}]}
        subscribers = list(get_resource_service("subscribers").get(req=None, lookup=query))

        if len(subscribers) == 0:
            raise SuperdeskApiError.badRequestError(message=_("No active subscribers found!"))

        if is_genre(article, BROADCAST_GENRE):
            digital_subscribers = list(self.digital(subscribers))
            if len(digital_subscribers) > 0:
                raise SuperdeskApiError.badRequestError(_("Only wire subscribers can receive broadcast stories!"))

        return subscribers

    def _validate_article(self, article_id, article_version):
        article = get_resource_service(ARCHIVE).find_one(req=None, _id=article_id)

        if app.config.get("CORRECTIONS_WORKFLOW") and article.get(ITEM_STATE) == "correction":
            publish_service = get_resource_service("published")
            article = publish_service.find_one(req=None, guid=article.get("guid"), state="being_corrected")

        if not article:
            raise SuperdeskApiError.badRequestError(message=_("Story couldn't be found!"))

        if article[ITEM_TYPE] not in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED]:
            raise SuperdeskApiError.badRequestError(message=_("Only text stories can be resent!"))

        if article.get(ITEM_STATE) not in [
            CONTENT_STATE.PUBLISHED,
            CONTENT_STATE.CORRECTED,
            CONTENT_STATE.KILLED,
            CONTENT_STATE.BEING_CORRECTED,
        ]:
            raise SuperdeskApiError.badRequestError(
                message=_("Only published, corrected or killed stories can be resent!")
            )

        if article[config.VERSION] != article_version:
            raise SuperdeskApiError.badRequestError(
                message=_("Please use the newest version {version} to resend!").format(version=article[config.VERSION])
            )

        if article.get("rewritten_by"):
            raise SuperdeskApiError.badRequestError(message=_("Updated story cannot be resent!"))

        return article



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import re
from superdesk.text_utils import get_text
from flask_babel import lazy_gettext

p = re.compile("(?i)(?<=[.?!])\\S+(?=[a-z])")


def populate(item, **kwargs):
    """Populate the abstract field with the first sentence of the body"""

    # get the list of sentences of the body
    if not item.get("body_html", None):
        item["abstract"] = "No body found to use for abstract..."
    else:
        sentences = p.split(item["body_html"])

        # chop the first sentence to size for abstract (64)
        if sentences and len(sentences) > 0:
            item["abstract"] = get_text(sentences[0][:64]).strip()

    return item


name = "populate_abstract"
label = lazy_gettext("Populate Abstract")
order = 3
shortcut = "a"
callback = populate
desks = ["POLITICS"]
access_type = "frontend"
action_type = "direct"
replace_type = "keep-style-replace"



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from flask_babel import lazy_gettext


def validate(item, **kwargs):
    """Checks if item has take_key value"""

    # validation
    if not item.get("anpa_take_key", "").strip():
        raise KeyError("Take key cannot be empty!")

    return item


name = "take_key_validator"
label = lazy_gettext("validate take key")
callback = validate
access_type = "backend"
action_type = "direct"



import unittest
from decimal import Decimal

from superdesk.macros.currency_usd_to_cad import usd_to_cad


class CurrencyTestCase(unittest.TestCase):
    def test_usd_to_cad(self):
        item = {"body_html": "$100"}
        res, diff = usd_to_cad(item, rate=Decimal(2))
        self.assertTrue("$100" in diff)
        self.assertEqual(diff["$100"], "$100 (CAD 200)")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import socket
import imaplib

from flask import current_app as app
from flask_babel import lazy_gettext as l_
from superdesk.errors import IngestEmailError
from superdesk.io.registry import register_feeding_service, register_feeding_service_parser
from superdesk.io.feeding_services import FeedingService
from superdesk.upload import url_for_media


class EmailFeedingService(FeedingService):
    """
    Feeding Service class which can read the article(s) from a configured mail box.
    """

    NAME = "email"

    ERRORS = [
        IngestEmailError.emailError().get_error_description(),
        IngestEmailError.emailLoginError().get_error_description(),
    ]

    label = "Email"

    fields = [
        {
            "id": "server",
            "type": "text",
            "label": l_("Email Server"),
            "placeholder": "Email Server",
            "required": True,
            "errors": {6003: "Server not found.", 6002: "Unexpected server response"},
        },
        {
            "id": "port",
            "type": "text",
            "label": l_("Email Server Port"),
            "placeholder": "Email Server Port",
            "required": True,
            "default": "993",
        },
        {"id": "user", "type": "text", "label": l_("User"), "placeholder": "User", "required": True},
        {
            "id": "password",
            "type": "password",
            "label": l_("Password"),
            "placeholder": "Password",
            "required": True,
            "errors": {6000: "Authentication error."},
        },
        {
            "id": "mailbox",
            "type": "text",
            "label": l_("Mailbox"),
            "placeholder": "Mailbox",
            "required": True,
            "errors": {6004: "Authentication error."},
        },
        {"id": "formatted", "type": "boolean", "label": l_("Formatted Email Parser"), "required": True},
        {"id": "filter", "type": "text", "label": l_("Filter"), "placeholder": "Filter", "required": False},
    ]

    def _test(self, provider):
        self._update(provider, update=None, test=True)

    def authenticate(self, provider: dict, config: dict) -> imaplib.IMAP4_SSL:
        server = config.get("server", "")
        port = int(config.get("port", 993))
        try:
            socket.setdefaulttimeout(app.config.get("EMAIL_TIMEOUT", 10))
            imap = imaplib.IMAP4_SSL(host=server, port=port)
        except (socket.gaierror, OSError) as e:
            raise IngestEmailError.emailHostError(exception=e, provider=provider)

        try:
            imap.login(config.get("user", None), config.get("password", None))
        except imaplib.IMAP4.error:
            raise IngestEmailError.emailLoginError(imaplib.IMAP4.error, provider)

        return imap

    def _update(self, provider, update, test=False):
        config = provider.get("config", {})
        new_items = []

        try:
            imap = self.authenticate(provider, config)

            try:
                rv, data = imap.select(config.get("mailbox", None), readonly=False)
                if rv != "OK":
                    raise IngestEmailError.emailMailboxError()
                try:
                    rv, data = imap.search(None, config.get("filter", "(UNSEEN)"))
                    if rv != "OK":
                        raise IngestEmailError.emailFilterError()
                    for num in data[0].split():
                        rv, data = imap.fetch(num, "(RFC822)")
                        if rv == "OK" and not test:
                            try:
                                parser = self.get_feed_parser(provider, data)
                                new_items.append(parser.parse(data, provider))
                                rv, data = imap.store(num, "+FLAGS", "\\Seen")
                            except IngestEmailError:
                                continue
                finally:
                    imap.close()
            finally:
                imap.logout()
        except IngestEmailError:
            raise
        except Exception as ex:
            raise IngestEmailError.emailError(ex, provider)
        return new_items

    def prepare_href(self, href, mimetype=None):
        return url_for_media(href, mimetype)


register_feeding_service(EmailFeedingService)
register_feeding_service_parser(EmailFeedingService.NAME, "email_rfc822")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from io import BytesIO
from unittest import mock

from superdesk.tests import TestCase
from superdesk import get_resource_service
from ..media import get_picture_fixture

from apps.picture_crop import get_crop_size


def get_file_mock(rendition, item):
    filename = get_picture_fixture()
    with open(filename, "rb") as imgfile:
        data = BytesIO(imgfile.read())
    return data


def crop_image_mock(content, file_name, cropping_data, exact_size=None, image_format=None):
    setattr(content, "width", 1)
    setattr(content, "height", 1)
    return True, content


def media_put_mock(content, filename=None, content_type=None, metadata=None, resource=None, **kwargs):
    return "foo"


class PictureCropServiceTest(TestCase):
    @mock.patch("apps.picture_crop.get_file", side_effect=get_file_mock)
    @mock.patch("apps.picture_crop.crop_image", side_effect=crop_image_mock)
    def test_crop_image_copies_metadata(self, get_file_function, crop_image_function):
        service = get_resource_service("picture_crop")
        images = [
            {
                "item": {"renditions": {"original": {"mimetype": "image/jpeg"}}},
                "crop": {"CropLeft": 10, "CropRight": 100, "CropTop": 10, "CropBottom": 100},
            }
        ]

        with mock.patch.object(self.app.media, "put", return_value="foo"):
            service.post(images)

        self.assertEqual(images[0]["metadata"].get("datetime"), '"2015:07:06 16:30:23"')
        self.assertEqual(images[0]["metadata"].get("exifimagewidth"), "400")
        self.assertEqual(images[0]["metadata"].get("exifimageheight"), "300")

    def test_get_crop_size_fixes_crop_aspect_ratio(self):
        crop_data = {"CropLeft": 0, "CropRight": 1620, "CropTop": 0, "CropBottom": 1230, "width": 800, "height": 600}
        get_crop_size(crop_data)

        crop_width = crop_data["CropRight"] - crop_data["CropLeft"]
        crop_height = crop_data["CropBottom"] - crop_data["CropTop"]

        crop_ratio = crop_height / crop_width
        size_ratio = crop_data["height"] / crop_data["width"]

        self.assertEqual(round(crop_ratio, 3), round(size_ratio, 3))

    def test_get_crop_size_uses_full_picture(self):
        crop_data = {
            "width": 1290,
            "height": 490,
            "CropTop": 59,
            "CropRight": 1300,
            "CropBottom": 554,
            "CropLeft": 0,
        }

        crop_size = get_crop_size(crop_data)
        self.assertEqual(1290, crop_size["width"])
        self.assertEqual(490, crop_size["height"])
        self.assertEqual(0, crop_data["CropLeft"])
        self.assertEqual(1300, crop_data["CropRight"])
        self.assertEqual(59, crop_data["CropTop"])
        self.assertGreaterEqual(554, crop_data["CropBottom"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk


class ValidatorsResource(superdesk.Resource):
    schema = {
        "_id": {"type": "string", "required": True, "unique": True},
        "act": {"type": "string", "required": True},
        "type": {"type": "string", "required": True},
        "embedded": {"type": "boolean", "required": False},
        "schema": {
            "type": "dict",
            "required": False,
            "schema": {},
            "allow_unknown": True,
        },
    }

    resource_methods = ["POST"]
    item_methods = []


class ValidatorsService(superdesk.Service):
    pass



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from datetime import datetime, timedelta

from superdesk.tests import TestCase
from superdesk.io.commands.update_ingest import is_new_version


class UtilsTestCase(TestCase):
    def test_is_new_version(self):
        self.assertTrue(is_new_version({"version": 2}, {"version": 1}))
        self.assertTrue(
            is_new_version({"versiocreated": datetime.now()}, {"versioncreated": datetime.now() - timedelta(days=1)})
        )
        self.assertTrue(is_new_version({"version": "10"}, {"version": "2"}))

        self.assertFalse(is_new_version({"version": 1}, {"version": 1}))
        self.assertFalse(is_new_version({"version": "123"}, {"version": "123"}))
        self.assertFalse(is_new_version({"versioncreated": datetime.now()}, {"versioncreated": datetime.now()}))

    def test_is_new_version_content(self):
        self.assertTrue(is_new_version({"headline": "foo"}, {}))
        self.assertTrue(is_new_version({"headline": "foo"}, {"headline": "bar"}))
        self.assertTrue(
            is_new_version(
                {"renditions": {"original": {"href": "foo"}}},
                {"renditions": {"original": {"href": "bar"}}},
            )
        )
        self.assertTrue(
            is_new_version(
                {"subject": [{"name": "foo", "qcode": "foo"}]},
                {"subject": [{"name": "bar", "qcode": "bar"}]},
            )
        )

        self.assertFalse(is_new_version({}, {}))
        self.assertFalse(is_new_version({"headline": "foo"}, {"headline": "foo", "source": "test"}))
        self.assertFalse(
            is_new_version(
                {"renditions": {"original": {"href": "foo"}}},
                {"renditions": {"original": {"href": "foo"}}},
            )
        )
        self.assertFalse(
            is_new_version(
                {"subject": [{"name": "foo", "qcode": "foo"}]},
                {"subject": [{"name": "foo", "qcode": "foo"}]},
            )
        )

    def test_is_new_version_ignores_expiry(self):
        yesterday = datetime.now() - timedelta(days=1)
        self.assertFalse(
            is_new_version(
                {"headline": "foo", "firstcreated": None, "expiry": datetime.now()},
                {"headline": "foo", "firstcreated": yesterday, "expiry": yesterday},
            )
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import re
import logging

from superdesk.errors import ParserError
from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import FileFeedParser
from superdesk.metadata.utils import generate_guid
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, GUID_TAG
from superdesk.utc import utcnow

logger = logging.getLogger(__name__)


class IPTC7901FeedParser(FileFeedParser):
    """
    Feed Parser which can parse if the feed is in IPTC 7901 format.
    """

    NAME = "iptc7901"

    label = "IPTC 7901 Parser"

    def can_parse(self, file_path):
        try:
            with open(file_path, "rb") as f:
                lines = [line for line in f]
                return re.match(
                    b"\x01([a-zA-Z]*)([0-9]*) (.) ([A-Z]{1,3}) ([0-9]*) ([a-zA-Z0-9 ]*)", lines[0], flags=re.I
                )
        except Exception:
            return False

    def parse(self, file_path, provider=None):
        try:
            item = {ITEM_TYPE: CONTENT_TYPE.TEXT, "guid": generate_guid(type=GUID_TAG), "versioncreated": utcnow()}

            with open(file_path, "rb") as f:
                lines = [line for line in f]
            # parse first header line
            m = re.match(b"\x01([a-zA-Z]*)([0-9]*) (.) ([A-Z]{1,3}) ([0-9]*) ([a-zA-Z0-9 ]*)", lines[0], flags=re.I)
            if m:
                item["original_source"] = m.group(1).decode("latin-1", "replace")
                item["ingest_provider_sequence"] = m.group(2).decode()
                item["priority"] = self.map_priority(m.group(3).decode())
                item["anpa_category"] = [{"qcode": self.map_category(m.group(4).decode())}]
                item["word_count"] = int(m.group(5).decode())

            inHeader = True
            inText = False
            inNote = False
            for line in lines[1:]:
                # STX starts the body of the story
                if line[0:1] == b"\x02":
                    # pick the rest of the line off as the headline
                    item["headline"] = line[1:].decode("latin-1", "replace").rstrip("\r\n")
                    item["body_html"] = ""
                    inText = True
                    inHeader = False
                    continue
                # ETX denotes the end of the story
                if line[0:1] == b"\x03":
                    break
                if inText:
                    if (
                        line.decode("latin-1", "replace").find("The following information is not for publication") != -1
                        or line.decode("latin-1", "replace").find(
                            "The following information is not intended for publication"
                        )
                        != -1
                    ):
                        inNote = True
                        inText = False
                        item["ednote"] = ""
                        continue
                    item["body_html"] += line.decode("latin-1", "replace")
                if inNote:
                    item["ednote"] += line.decode("latin-1", "replace")
                    continue
                if inHeader:
                    if "slugline" not in item:
                        item["slugline"] = ""
                    item["slugline"] += line.decode("latin-1", "replace").rstrip("/\r\n")
                    continue

            return item
        except Exception as ex:
            raise ParserError.IPTC7901ParserError(exception=ex, provider=provider)

    def map_category(self, source_category):
        if source_category == "x" or source_category == "X":
            return "i"
        else:
            return source_category


register_feed_parser(IPTC7901FeedParser.NAME, IPTC7901FeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015, 2016, 2017 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk.tests import TestCase
from superdesk import get_resource_service


class ItemDeleteTestCase(TestCase):
    """Tests that item_version documents are deleted when the items document is deleted"""

    def setUp(self):
        self.app.data.insert(
            "items",
            [
                {"_id": "item1", "type": "text"},
                {"_id": "item2", "type": "text"},
                {"_id": "item3", "type": "text"},
                {"_id": "item4", "type": "text"},
                {"_id": "item5", "type": "text"},
            ],
        )
        self.app.data.insert(
            "items_versions",
            [
                {"_id": "ver1", "type": "text", "_id_document": "item1"},
                {"_id": "ver2", "type": "text", "_id_document": "item2"},
                {"_id": "ver3", "type": "text", "_id_document": "item3"},
                {"_id": "ver4", "type": "text", "_id_document": "item4"},
                {"_id": "ver5", "type": "text", "_id_document": "item5"},
            ],
        )

    def test_event_fired(self):
        items_service = get_resource_service("items")
        items_versions_service = get_resource_service("items_versions")

        item_ids = ["item1", "item2", "item3", "item4", "item5"]
        ver_ids = ["ver1", "ver2", "ver3", "ver4", "ver5"]

        items_service.delete_action(lookup={"_id": {"$in": item_ids}})
        for item_id in item_ids:
            item = items_service.find_one(req=None, _id=item_id)
            self.assertIsNone(item)

        for ver_id in ver_ids:
            item_version = items_versions_service.find_one(req=None, _id=ver_id)
            self.assertIsNone(item_version)
